/**
 * @version working
 * @author Thomas kriegelstein
 * $Id: Reinit.java,v 1.10 2003/11/05 09:13:51 tk4 Exp $
 */
/** Vorgehen zum Minimalbackup der Datenbank.
 * Zu speichern:
 * 1. Pseudonymtyp personal
 * 2. passende Pseudonyme
 * 3. deren Public Keys + Secret Keys
 * 4. Partnertyp Identittstreuhnder
 * 5. dessen Public Key und Selbstzertifikat
 * 6. zu den Public Keys die Certificate
 * 7. Die Regelketten
 * 8. Die Configurations
 * 9. Eigene Daten (owner==null)
 * Dann Init.initialize
 * Anschliessend alles wieder eintragen.
 * Immer Ergebnisse zwischenspeichern. Insbesondere for initialize -> "reinit.o"
 **/
package cebit;

import psman.dbi.DBI;
import idman.dbi.*;
import idman.mngt.Management;
import psman.data.*;
import java.io.*;
import java.util.*;

/**
 * Klasse dient zum Rcksetzen auf einen Anfangszustand.
 * Alle Transaktionen werden gelscht.
 * @author Thomas Kriegelstein
 * @version $Id: Reinit.java,v 1.10 2003/11/05 09:13:51 tk4 Exp $
 */
public class Reinit implements Serializable {
    protected           String         spersonal = "personal";
    protected transient PseudonymType   personal = null;
    
    protected           String[]     spseudonyms = {};
    protected transient Pseudonym[]   pseudonyms = {};
    
    // index in publickeys ist index in pseudonyms!
    protected           byte[][][]   spublickeys = {};
    protected transient PublicKey[]   publickeys = {};
    
    // index in secretkeys ist index in pseudonyms!
    protected           byte[][][]   ssecretkeys = {};
    protected transient SecretKey[]   secretkeys = {};
    
    protected           String          strustee = null;
    protected transient Type             trustee = null;
    
    protected           String[]       strustees = {};
    protected transient TPartner[]      trustees = {};
    
    // index in trustkeys ist index in trustees!
    protected           byte[][][]    strustkeys = {};
    protected transient PublicKey[]    trustkeys = {};
    
    // index in trustcerts ist index in trustees!
    protected           byte[][][]   strustcerts = {};
    protected transient Certificate[] trustcerts = {};
    
    // index in certs ist index in pseudonyms!
    protected           byte[][][]        scerts = {};
    protected transient Certificate[]      certs = {};
    // index in cindexs ist index in pseudonyms! wert in cindexs ist index in trustees!
    protected           int[]            cindexs = {};
    // index in selfcerts ist index in pseudonyms!
    protected           byte[][][]    sselfcerts = {};
    protected transient Certificate[]  selfcerts = {};
    
    protected           String[][]        sconfs = {};
    protected transient Configuration[]    confs = {};
    
    protected           String[]         schains = {};
    protected transient Chain[]           chains = {};
    
    // index in rules ist index in chains! indexindex in rules ist rang
    protected           String[][][]      srules = {};
    protected transient Rule[][]           rules = {};
    
    protected           byte[][][]        sdatas = {};
    protected transient DataField[]        datas = {};
    
    protected transient boolean tested = true;
    
    public void reinit(java.util.Properties p) throws Exception {
        DBI dbi = Database.getDBI(p);
        Type t = null;
        try {
            t = dbi.selectType(idman.mngt.Management.IDENTITY_CA);
        } catch (Exception e) {
            Init.initialize(p);
	    dbi = Database.getDBI(p);
        }
        if (t==null) {
            restore();
        } else {
            trustee    = t;
            trustees   = dbi.selectTPartner(trustee);
            trustkeys  = new PublicKey[trustees.length];
            trustcerts = new Certificate[trustees.length];
            
            personal   = dbi.selectPseudonymType("personal");
            pseudonyms = dbi.selectPseudonym(personal);
            
            publickeys = new PublicKey[pseudonyms.length];
            secretkeys = new SecretKey[pseudonyms.length];
            certs      = new Certificate[pseudonyms.length];
            selfcerts  = new Certificate[pseudonyms.length];
            cindexs    = new int[pseudonyms.length];
            
            for (int i=0; i<trustees.length; i++) {
                trustkeys[i] =trustees[i].getPublicKey();
                trustcerts[i]=dbi.selectCertificate(trustkeys[i].getID());
            }
            for (int i=0; i<pseudonyms.length; i++) {
                secretkeys[i]=pseudonyms[i].getMasterKey();
                publickeys[i]=dbi.selectPublicKey(secretkeys[i].getID());
            }
            for (int i=0; i<pseudonyms.length; i++) {
                for (int j=0; j<trustees.length && certs[i]==null; j++) {
                    byte[] id = Certificate.computeID(publickeys[i].getEncoded(), trustees[j].getPublicKey().getEncoded());
                    certs[i] = dbi.selectCertificate(id);
                    if (certs[i]!=null) cindexs[i]=j;
                }
                byte[] cid = Certificate.computeID(publickeys[i].getEncoded(), publickeys[i].getEncoded());
                selfcerts[i] = dbi.selectCertificate(cid);
            }
            
            confs = dbi.selectConfiguration();
            
            chains = dbi.selectChain();
            rules = new Rule[chains.length][];
            
            for (int i=0; i<chains.length; i++) {
                Rule[] tmp = new Rule[2];
                int length=0;
                for (int rank=1;;rank++,length++) {
                    Rule r = dbi.selectRule(chains[i].getID(), rank);
                    if (r!=null) {
                        // Array vergroessern
                        if (tmp.length==rank-1) {
                            Rule[] tmptmp = new Rule[tmp.length*2];
                            System.arraycopy(tmp,0,tmptmp,0,tmp.length);
                            tmp = tmptmp;
                        }
                        tmp[rank-1]=r;
                    } else break;
                }
                Rule[] rmp = new Rule[length];
                System.arraycopy(tmp,0,rmp,0,length);
                rules[i] = rmp;
            }
            
            datas = dbi.selectDataField((TPartner)null);
            // Alles fertig ausgelesen
        }
        // -> Abspeichern
        try {
            save();
            if (!tested) restore();
        } catch (IOException ioe) {
            System.err.println("Backup failed: "+ioe);
        }
        // Datenbank neu anlegen
        // HANDLE WITH CARE!
        if (tested) {
            Init.initialize(p);
            // Daten wieder eintragen
            // reconnect, weil Init.initialize die Verbindung schliesst
            dbi = Database.getDBI(p);
            // DataFields
            for (int i=0; i<datas.length; i++) {
                //System.out.println("Inserting: "+datas[i].getName()+" ("+new String(datas[i].getValue())+")");
                //dbi.insert(datas[i]);
            }
            // Regelkette
            for (int i=0; i<chains.length; i++) {
                chains[i] = new Chain(chains[i].getName());
                dbi.insert(chains[i]);
                for (int j=0; j<rules[i].length; j++) {
                    rules[i][j] = new Rule(chains[i], j+1, rules[i][j].getCondition(), rules[i][j].getAction(), rules[i][j].getParameter());
                    dbi.insert(rules[i][j]);
                }
            }
            // Configurationen
            for (int i=0; i<confs.length; i++) {
                confs[i]=new Configuration(confs[i].getID(), confs[i].getValue());
                dbi.insert(confs[i]);
            }
            // TPartners
            for (int i=0; i<trustees.length; i++) {
                trustees[i]=new TPartner(trustees[i].getName(),trustkeys[i],(Person)null);
                dbi.insert(trustkeys[i]);
                dbi.insert(trustcerts[i]);
                dbi.insert(trustkeys[i], new Certificate[] {trustcerts[i]});
                dbi.insert(trustcerts[i], trustkeys[i]);
                dbi.insert(trustees[i]);
            }
            // Pseudonyms
            personal = dbi.selectPseudonymType(personal.getName());
            for (int i=0; i<pseudonyms.length; i++) {
                dbi.insert(publickeys[i]);
                dbi.insert(secretkeys[i]);
                pseudonyms[i]=new Pseudonym(pseudonyms[i].getName(), secretkeys[i]);
                dbi.insert(pseudonyms[i]);
                dbi.insert(pseudonyms[i], new PseudonymType[] { personal });
                dbi.insert(certs[i]);
                dbi.insert(selfcerts[i]);
                dbi.insert(publickeys[i], new Certificate[] { certs[i], selfcerts[i] });
                dbi.insert(certs[i], trustkeys[cindexs[i]]);
                dbi.insert(selfcerts[i], publickeys[i]);
            }
            // Markiert Abschluss der Reinitialisierung
            trustee = new Type(idman.mngt.Management.IDENTITY_CA);
            dbi.insert(trustee);
            // Nachbereitung, wenn die schiefgeht -> Mist
            for (int i=0; i<trustees.length; i++) {
                dbi.insert(trustees[i], new Type[] { trustee });
            }
            Database.ungetDBI();
            System.out.println("All done.");
        }
    }
    public void restore() throws IOException, ClassNotFoundException {
        System.out.println("Restoring.");
        // reload
        Reinit tmp = (Reinit)(new ObjectInputStream(new FileInputStream("reinit.o")).readObject());
        this.personal   = new PseudonymType(tmp.spersonal);
        this.pseudonyms = new Pseudonym[tmp.spseudonyms.length];
        this.publickeys = new PublicKey[tmp.spublickeys.length];
        this.secretkeys = new SecretKey[tmp.ssecretkeys.length];
        this.trustee    = new Type(Management.IDENTITY_CA);
        this.trustees   = new TPartner[tmp.strustees.length];
        this.trustkeys  = new PublicKey[tmp.strustkeys.length];
        this.trustcerts = new Certificate[tmp.strustcerts.length];
        this.certs      = new Certificate[tmp.scerts.length];
        this.selfcerts  = new Certificate[tmp.sselfcerts.length];
        this.cindexs    = tmp.cindexs;
        this.confs      = new Configuration[tmp.sconfs.length];
        this.chains     = new Chain[tmp.schains.length];
        this.rules      = new Rule[tmp.srules.length][];
        this.datas      = new DataField[tmp.sdatas.length];
        for (int i=0; i<chains.length; i++) {
            chains[i] = new Chain(tmp.schains[i]);
            rules[i]  = new Rule[tmp.srules[i].length];
            for (int j=0; j<tmp.srules[i].length; j++) {
                rules[i][j] = new Rule(chains[i], tmp.srules[i][j][0], tmp.srules[i][j][1], tmp.srules[i][j][2]);
            }
        }
        for (int i=0; i<confs.length; i++) {
            confs[i] = new Configuration(tmp.sconfs[i][0], tmp.sconfs[i][1]);
        }
        for (int i=0; i<pseudonyms.length; i++) {
            certs[i]      = new Certificate(tmp.scerts[i][0], tmp.scerts[i][1]);
            selfcerts[i]  = new Certificate(tmp.sselfcerts[i][0], tmp.sselfcerts[i][1]);
            publickeys[i] = new PublicKey(tmp.spublickeys[i][0], tmp.spublickeys[i][1]);
            secretkeys[i] = new SecretKey(tmp.ssecretkeys[i][0], tmp.ssecretkeys[i][1]);
            pseudonyms[i] = new Pseudonym(tmp.spseudonyms[i], secretkeys[i]);
        }
        for (int i=0; i<trustees.length; i++) {
            trustkeys[i]  = new PublicKey(tmp.strustkeys[i][0], tmp.strustkeys[i][1]);
            trustcerts[i] = new Certificate(tmp.strustcerts[i][0], tmp.strustcerts[i][1]);
            trustees[i]   = new TPartner(tmp.strustees[i], trustkeys[i], null);
        }
        java.util.Date date = new java.util.Date();
        for (int i=0; i<datas.length; i++) {
            datas[i] = new DataField(null, new String(tmp.sdatas[i][0]), tmp.sdatas[i][1], date, date);
        }
    }
    public void save() throws IOException {
        spseudonyms = new String[pseudonyms.length];
        spublickeys = new byte[pseudonyms.length][][];
        ssecretkeys = new byte[pseudonyms.length][][];
        scerts      = new byte[pseudonyms.length][][];
        sselfcerts  = new byte[pseudonyms.length][][];
        strustee    = trustee.getName();
        strustees   = new String[trustees.length];
        strustkeys  = new byte[trustees.length][][];
        strustcerts = new byte[trustees.length][][];
        sconfs      = new String[confs.length][];
        schains     = new String[chains.length];
        srules      = new String[chains.length][][];
        sdatas      = new byte[datas.length][][];
        for (int i=0; i<pseudonyms.length; i++) {
            spseudonyms[i] = pseudonyms[i].getName();
            spublickeys[i] = new byte[][] { publickeys[i].getID(), publickeys[i].getEncoded() };
            ssecretkeys[i] = new byte[][] { secretkeys[i].getID(), secretkeys[i].getEncoded() };
            scerts[i]      = new byte[][] { certs[i].getID(), certs[i].getValue() };
            sselfcerts[i]  = new byte[][] { selfcerts[i].getID(), selfcerts[i].getValue() };
        }
        for (int i=0; i<trustees.length; i++) {
            strustees[i]   = trustees[i].getName();
            strustkeys[i]  = new byte[][] { trustkeys[i].getID(), trustkeys[i].getEncoded() };
            strustcerts[i] = new byte[][] { trustcerts[i].getID(), trustcerts[i].getValue() };
        }
        for (int i=0; i<confs.length; i++) {
            sconfs[i]  = new String[] { confs[i].getID(), confs[i].getValue() };
        }
        for (int i=0; i<chains.length; i++) {
            schains[i] = chains[i].getName();
            srules[i]  = new String[rules[i].length][];
            for (int j=0; j<rules[i].length; j++) {
                srules[i][j]=new String[] {rules[i][j].getCondition(), rules[i][j].getAction(), rules[i][j].getParameter()};
            }
        }
        for (int i=0; i<datas.length; i++) {
            sdatas[i] = new byte[][] { datas[i].getName().getBytes(), datas[i].getValue() };
        }
        new ObjectOutputStream(new FileOutputStream("reinit.o")).writeObject(this);
    }
    public static void main(String[] args) throws Exception {
        if (args.length!=1) {
            System.out.println("Specify config-file");
        } else {
            Properties p = new Properties();
            p.load(new FileInputStream(args[0]));
            convert(p);
            (new Reinit()).reinit(p);
        }
    }
    private static void convert(Properties p) {
        for(Enumeration e = p.propertyNames(); e.hasMoreElements(); ) {
            String name = (String)e.nextElement();
            if (name.startsWith("dbi.")) {
                String val = p.getProperty(name);
                p.setProperty(name, val);
	    } else if (name.startsWith("cebit.server.idm.")) {
                String val = p.getProperty(name);
                p.setProperty(name.substring(17), val);
            } else if (name.startsWith("cebit.proxy.idm.")) {
		String val = p.getProperty(name);
		p.setProperty(name.substring(16), val);
	    }
        }
    }
}
