/**
 * Copyright (c) 2002 Sebastian Clau, IDM Team
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package cebit.proxy;

import org.w3c.jigsaw.http.httpd;

import org.w3c.jigsaw.config.PropertySet;
import org.w3c.tools.resources.Attribute;
import org.w3c.tools.resources.StringAttribute;
import org.w3c.tools.resources.FileAttribute;
import org.w3c.tools.resources.IntegerAttribute;
import org.w3c.tools.resources.AttributeRegistry;
import org.w3c.tools.resources.BooleanAttribute;

/**
 * This class holds the property set needed for IDM proxy support of jigsaw.
 */
public class ProxyProperties extends PropertySet {

    /** default properties name */
    public static final String DEFAULT_NAME = "IDMProxy";

    /** Database username key */
    public static final String DBI_USERNAME_P = "cebit.proxy.idm.dbi.username";
    /** Database hostname key */
    public static final String DBI_HOSTNAME_P = "cebit.proxy.idm.dbi.hostname";
    /** Database Filename key */
    public static final String DBI_FILENAME_P = "cebit.proxy.idm.dbi.filename";
    /** Database password key */
    public static final String DBI_PASSWORD_P = "cebit.proxy.idm.dbi.password";
    /** Database name key */
    public static final String DBI_DATABASE_P = "cebit.proxy.idm.dbi.database";
    /** Database port key */
    public static final String DBI_PORT_P = "cebit.proxy.idm.dbi.port";
    /** Database driverClass key */
    public static final String DBI_DRIVERCLASS_P = "cebit.proxy.idm.dbi.driverClass";
    /** Database concelation */
    public static final String DBI_CONCEAL_P = "cebit.proxy.idm.dbi.conceal";
    /** Database passphrase for concealation */
    public static final String DBI_PASSPHRASE_P = "cebit.proxy.idm.dbi.passphrase";
    /** Personal Pseudonym Name key */
    public static final String IDM_PERSONAL_PSEUDONYM_P = "cebit.proxy.idm.personalpseudonym";

    protected static int ATTR_DBI_USERNAME = -1;
    protected static int ATTR_DBI_HOSTNAME = -1;
    protected static int ATTR_DBI_FILENAME = -1;
    protected static int ATTR_DBI_PASSWORD = -1;
    protected static int ATTR_DBI_DATABASE = -1;
    protected static int ATTR_DBI_PORT = -1;
    protected static int ATTR_DBI_DRIVERCLASS = -1;
    protected static int ATTR_DBI_CONCEAL = -1;
    protected static int ATTR_DBI_PASSPHRASE = -1;
    protected static int ATTR_IDM_PERSONAL_PSEUDONYM = -1;
    /**
     * static initializer for IDM properties
     */
    static {
        Class cls = null;
        Attribute a = null;
        try {
            cls = Class.forName("cebit.proxy.ProxyProperties");
        }
        catch (Exception ex) {
            ex.printStackTrace();
            System.exit(1);
        }
        a = new StringAttribute(DBI_USERNAME_P, null, Attribute.EDITABLE);
        ATTR_DBI_USERNAME = AttributeRegistry.registerAttribute(cls, a);
        a = new StringAttribute(DBI_HOSTNAME_P, null, Attribute.EDITABLE);
        ATTR_DBI_HOSTNAME = AttributeRegistry.registerAttribute(cls, a);
        a = new FileAttribute(DBI_FILENAME_P, null, Attribute.EDITABLE);
        ATTR_DBI_FILENAME = AttributeRegistry.registerAttribute(cls, a);
        a = new StringAttribute(DBI_PASSWORD_P, null, Attribute.EDITABLE);
        ATTR_DBI_PASSWORD = AttributeRegistry.registerAttribute(cls, a);
        a = new StringAttribute(DBI_DATABASE_P, null, Attribute.EDITABLE);
        ATTR_DBI_DATABASE = AttributeRegistry.registerAttribute(cls, a);
        a = new IntegerAttribute(DBI_PORT_P, null, Attribute.EDITABLE);
        ATTR_DBI_PORT = AttributeRegistry.registerAttribute(cls, a);
        // default could be "dbi.driverClass=psman.dbi.XML"
        a = new StringAttribute(DBI_DRIVERCLASS_P, null, Attribute.EDITABLE);
        ATTR_DBI_DRIVERCLASS = AttributeRegistry.registerAttribute(cls, a);
        a = new BooleanAttribute(DBI_CONCEAL_P, null, Attribute.EDITABLE);
        ATTR_DBI_CONCEAL = AttributeRegistry.registerAttribute(cls, a);
        a = new StringAttribute(DBI_PASSPHRASE_P, null, Attribute.EDITABLE);
        ATTR_DBI_PASSPHRASE = AttributeRegistry.registerAttribute(cls, a);
        a = new StringAttribute(IDM_PERSONAL_PSEUDONYM_P, null, Attribute.EDITABLE);
        ATTR_IDM_PERSONAL_PSEUDONYM = AttributeRegistry.registerAttribute(cls, a);
    }

    /**
     * nameless constructor for IDM proxy properties
     * @param server reference to the current daemon
     */
    public ProxyProperties(httpd server) {
        super(DEFAULT_NAME, server);
    }

    /**
     * named constructor for IDM proxy properties
     * @param name the name for the daemon in question
     * @param server reference to the current daemon
     */
    public ProxyProperties(String name, httpd server) {
        super(name, server);
    }

}
