package cebit.proxy.gui;

import javax.swing.*;
import javax.swing.border.EtchedBorder;

import java.awt.*;
import java.io.File;
import java.beans.*;

/**
 * @author sc2
 *
 * Shows a preview of images in JFileChooser.
 */
public class ImagePreview extends JComponent implements PropertyChangeListener{

	ImageIcon image = null;
	File file = null;

	public ImagePreview(JFileChooser jfc) {
		setPreferredSize(new Dimension(90, 90));
		jfc.addPropertyChangeListener(this);
		this.setBorder(new EtchedBorder());
	}

	public void loadImage() {
		if (file == null) {
			image = null;
			return;
		}

		ImageIcon tmpImage = new ImageIcon(file.getPath());
		if (tmpImage != null) {
			if ((tmpImage.getIconWidth() != 64) || (tmpImage.getIconHeight() != 64)) {
				image = new ImageIcon(tmpImage.getImage().
					getScaledInstance(64, 64,
					Image.SCALE_DEFAULT));
			} else {
				image = tmpImage;
			}
		}
	}

	public void propertyChange(PropertyChangeEvent e) {
		boolean update = false;
		if (JFileChooser.DIRECTORY_CHANGED_PROPERTY.equals(e.getPropertyName())) {
			file = null;
			update = true;
		} else if (JFileChooser.SELECTED_FILE_CHANGED_PROPERTY.equals(e.getPropertyName())) {
			file = (File) e.getNewValue();
			update = true;
		}
		//Update preview
		if (update) {
			image = null;
			if (isShowing()) {
				loadImage();
				repaint();
			}
		}
	}

	public void paintComponent(Graphics g) {
		if (image == null) {
			loadImage();
		}
		if (image != null) {
			int x = getWidth()/2 - image.getIconWidth()/2;
			int y = getHeight()/2 - image.getIconHeight()/2;

			if (y < 10) y = 10;
			if (x < 10) x = 10;

			image.paintIcon(this, g, x, y);
		}
	}

    public static void main(String[] args) {}
}

