// GenericAuthFilter.java
// $Id: IDMAuthFilter.java,v 1.1 2003/10/16 14:59:56 tk4 Exp $
// (c) COPYRIGHT MIT and INRIA, 1996.
// Please first read the full copyright statement in file COPYRIGHT.html

package cebit.proxy.idm;

import org.w3c.tools.resources.ProtocolException;
import org.w3c.jigsaw.http.Client;
import org.w3c.jigsaw.http.HTTPException;
import org.w3c.jigsaw.http.Reply;
import org.w3c.jigsaw.http.Request;
import org.w3c.jigsaw.html.HtmlGenerator;
import org.w3c.jigsaw.auth.AuthFilter;
import org.w3c.tools.codec.Base64Decoder;
import org.w3c.tools.codec.Base64FormatException;

import org.w3c.www.http.HTTP;
import org.w3c.www.http.HttpChallenge;
import org.w3c.www.http.HttpCredential;
import org.w3c.www.http.HttpFactory;

public class IDMAuthFilter extends AuthFilter {
    
    /**
     * The challenge to issue to any client for Basic Authentication.
     */
    protected HttpChallenge challenge = null;
    
    /**
     * Authenticate the given request.
     * We first check for valid authentication information. If no
     * authentication is provided, than we try to map the IP address to some
     * of the ones we know about. If the IP address is not found, we challenge
     * the client for a password.
     * <p>If the IP address is found, than either our user entry requires an
     * extra password step (in wich case we challenge it), or simple IP
     * based authentication is enough, so we allow the request.
     * @param request The request to be authentified.
     * @exception org.w3c.tools.resources.ProtocolException if authentication
     * failed
     */
    
    public void authenticate(Request request)
    throws ProtocolException {
        // Internal requests always allowed:
        Client client = request.getClient() ;
        if ( client == null )
            return ;
        // Check authentication according to auth method:
        if ((request.hasAuthorization() && ! request.isProxy())
        || (request.isProxy() && request.hasProxyAuthorization())) {
            String user = null;
            String cookie = null;
            HttpCredential credential = null;
            
            credential = (request.isProxy()
            ? request.getProxyAuthorization()
            : request.getAuthorization());
            if ( ! credential.getScheme().equalsIgnoreCase("Basic") ) {
                String msg = ("Invalid authentication scheme \""
                + credential.getScheme()
                + " expecting \"Basic\"");
                throw new HTTPException(msg) ;
            }
            // Decode the credentials:
            String decoded = null ;
            cookie    = credential.getAuthParameter("cookie");
            try {
                Base64Decoder b  = new Base64Decoder(cookie) ;
                decoded          = b.processString() ;
            } catch (Base64FormatException e) {
                String msg = "Invalid BASE64 encoding of credentials." ;
                throw new HTTPException(msg) ;
            }
            // Get user and password:
            int icolon = decoded.indexOf(':') ;
            if ( (icolon > 0) && (icolon+1 < decoded.length()) ) {
                // ok, parse was find, check user:
                user     = decoded.substring(0, icolon) ;
            } else {
                String msg = "Invalid credentials syntax in " + decoded ;
                throw new HTTPException(msg) ;
            }
            
            request.setState(STATE_AUTHUSER, user);
            request.setState(STATE_AUTHTYPE, "Basic") ;
            return;
        }
        
        // Every possible scheme has failed for this request, emit an error
        Reply e = null;
        if ( request.isProxy() ) {
            e = request.makeReply(HTTP.PROXY_AUTH_REQUIRED);
            e.setProxyAuthenticate(challenge);
        } else {
            e = request.makeReply(HTTP.UNAUTHORIZED);
            e.setWWWAuthenticate(challenge);
        }
        HtmlGenerator g = new HtmlGenerator("Unauthorized");
        g.append("<h1>Unauthorized access</h1>"
        + "<p>You are denied access to this resource.");
        e.setStream(g);
        throw new HTTPException(e);
    }
    
    /**
     * Initialize the filter.
     */
    
    public void initialize(Object values[]) {
        super.initialize(values) ;
        // Initialize the filter challenge:
        challenge = HttpFactory.makeChallenge("Basic");
        challenge.setAuthParameter("realm", "Proxy");
    }
}
