package cebit.proxy.idm;

import idman.gui.LocalManagementGUI;
import idman.mngt.impl.LocalManagement;

import java.util.Properties;

import javax.swing.JFrame;

import org.ssonet.util.Preload;
import org.w3c.jigsaw.auth.AuthFilter;
import org.w3c.jigsaw.http.Request;
import org.w3c.jigsaw.http.httpd;
import org.w3c.jigsaw.proxy.ProxyFrame;
import org.w3c.tools.resources.FramedResource;
import org.w3c.util.ObservableProperties;

import cebit.proxy.Proxy;
import cebit.proxy.ProxyProperties;

/**
 * Thist Frame works like a normal proxy frame, except that it initializes
 * an IDM tool on first startup. The IDM tool is static, so that there is only
 * one IDM tool for all proxies.
 */
public class IDMProxyFrame extends ProxyFrame{
    private static boolean debug = false;
    private static boolean initialized = false;

    public org.w3c.www.protocol.http.Request dupRequest(Request request) {
        try {
            org.w3c.www.protocol.http.Request result = super.dupRequest(request);
            result.setState(AuthFilter.STATE_AUTHUSER, request.getState(AuthFilter.STATE_AUTHUSER));
            return result;
        } catch (Exception e) {
            return null;
        }
    }
    
    static {
        // preload security provider and mechanisms for SSONET
        Preload preload = new Preload();
        try{
            preload.join();
        }catch (Exception e){};
        try{
            idman.util.KeyProvider.start();
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    /**
     * Register this resource with the server, get the property set,
     * initialize the IDManager.
     */
    public void registerResource(FramedResource resource) {
        // do the parent registration
	super.registerResource(resource);
        if(debug) System.out.println("IDMProxyFrame: registerResource called.");
        // Register IDM properties
        try{
            ObservableProperties props = getResource().getServer().getProperties();
            synchronized(this.getClass()) {
                if (!initialized) {
                    httpd server = (httpd) getServer();
                    server.registerPropertySet(new ProxyProperties("IDMProxy", server));
                    if(debug) System.out.println("IDMProxyFrame: IDM-Properties registered.");
                    initialized = true;
                }
            }
            // initialize the Identity manager and save it into a static variable XXX
            // initialize IDM
            if(Proxy.idManager == null){
                Properties settings = new Properties();
                settings.setProperty("dbi.username",
                    props.getString(ProxyProperties.DBI_USERNAME_P, ""));
                settings.setProperty("dbi.hostname",
                    props.getString(ProxyProperties.DBI_HOSTNAME_P, ""));
                settings.setProperty("dbi.filename",
                    props.getString(ProxyProperties.DBI_FILENAME_P, ""));
                settings.setProperty("dbi.password",
                    props.getString(ProxyProperties.DBI_PASSWORD_P, ""));
                settings.setProperty("dbi.database",
                    props.getString(ProxyProperties.DBI_DATABASE_P, ""));
                settings.setProperty("dbi.port",
                    props.getString(ProxyProperties.DBI_PORT_P, ""));
                settings.setProperty("dbi.driverClass",
                    props.getString(ProxyProperties.DBI_DRIVERCLASS_P, ""));
                settings.setProperty("dbi.conceal",
                    props.getString(ProxyProperties.DBI_CONCEAL_P, ""));
                settings.setProperty("dbi.passphrase",
                    props.getString(ProxyProperties.DBI_PASSPHRASE_P, ""));
                settings.setProperty("mngt.certificate.subjectCN",
                    props.getString(ProxyProperties.IDM_PERSONAL_PSEUDONYM_P, ""));
                // set Properties for the IDManager
                if (debug) System.out.println(settings);
                // create IDManager
                Proxy.idManager  = new LocalManagement(settings);
                if (true) try {
                    // create IDManager GUI
                    JFrame frame = new JFrame("Proxy IDManagement");
                    Proxy.idManagerGUI = new LocalManagementGUI(Proxy.idManager, frame, settings);
                    frame.getContentPane().add(Proxy.idManagerGUI);
                    frame.setSize(400, 400);
                    frame.setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
                    frame.show();
                    if(debug) System.out.println("IDMProxyFrame: IDManager succesfully started");
                } catch (Error e) {
                    System.err.println("IDMProxyFrame: Could not open ID-Management GUI: "+e.getMessage());
                    if (debug) e.printStackTrace();
                }
                // set personal pseudonym
                Proxy.idManager.settings.setProperty("mngt.certificate.subjectCN",
                    props.getString(ProxyProperties.IDM_PERSONAL_PSEUDONYM_P, ""));
                idman.neg.NegotiationHandlerFactory.addNewNegotiationHandler("idman.neg.impl.RelevantNegotiationHandler","RELEVANT");
            }
        }catch(Exception e){
            e.printStackTrace();
        }
        if (debug) System.out.println(">>> registerResource: "+manager);
        // change properties, so manager really gets installed
        if (manager!=null) {
		ObservableProperties props = manager.getProperties();
		props.setProperty("org.w3c.jigsaw.idm.lastStart",(new java.util.Date()).toString());
        	manager = org.w3c.www.protocol.http.HttpIDMManager.getManager(props);
	}
        if (debug) System.out.println(">>> registerResource: "+manager);
    }

}
