package cebit.proxy.idm;

import idman.dbi.Database;

import java.io.*;
import java.security.Principal;
import java.security.cert.CertificateException;
import java.security.cert.CertificateFactory;
import java.security.cert.X509Certificate;
import java.util.*;

import javax.servlet.ServletConfig;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.ssonet.net.CryptoTools;
import org.w3c.jigsaw.proxy.ProxyFrame;
import org.w3c.tools.resources.FramedResource;
import org.w3c.tools.resources.ServerInterface;

import psman.data.*;
import psman.dbi.DBI;
import cebit.proxy.Proxy;

public class LoggingServlet extends HttpServlet {

    public static boolean debug = false;
    
    public static String SESSION_LAST_TIME_USED = "lastTimeUsed";
    public static String SESSION_LOCALE = "locale";
	public static String SESSION_RESOURCE_BUNDLE = "resourceBundle";
	public static String SESSION_SORTING_TYPE = "sortingType";
	public static String SESSION_SELECTED_ROLE = "selectedRole";
	public static String SESSION_SELECTED_PSEUDONYM = "selectedPseudonym";
	public static String SESSION_SELECTED_TRANSACTION = "selectedTransaction";
	public static String SESSION_SELECTED_TPARTNER = "selectedTPartner";
	public static String SESSION_SELECTED_DATA_FIELD_NAME = "selectedDataFieldName";
	public static String SESSION_SELECTED_DATA_FIELD_VALUE = "selectedDataFieldValue";
	
	public static String QUERY_SESSION_ID = "sessionId";
	public static String QUERY_LANG = "lang";
	public static String QUERY_FRAME = "frame";
	public static String QUERY_SORTING_TYPE = "sortingType";
	public static String QUERY_LIST_SELECTED = "listSelected";
	public static String QUERY_CONTENT_SELECTED = "contentSelected";
	public static String QUERY_DATA_TYPE = "dataType";
	

    private static Hashtable sessions = new Hashtable();
    private DBI dbi = null;
	static HashSet p3pNames = new HashSet();
	{
		p3pNames.add("#user.department");
		p3pNames.add("#user.name.prefix");
		p3pNames.add("#user.name.given");
		p3pNames.add("#user.name.family");
		p3pNames.add("#user.home-info.telecom.telephone.loccode");
		p3pNames.add("#user.home-info.telecom.telephone.number");
		p3pNames.add("#user.home-info.online.email");
		p3pNames.add("#user.home-info.postal.street");
		p3pNames.add("#user.home-info.postal.postalcode");
		p3pNames.add("#user.home-info.postal.city");
		p3pNames.add("#user.business-info.telecom.telephone.loccode");
		p3pNames.add("#user.business-info.telecom.telephone.number");
		p3pNames.add("#user.business-info.online.email");
		p3pNames.add("#user.business-info.postal.street");
		p3pNames.add("#user.business-info.postal.postalcode");
		p3pNames.add("#user.business-info.postal.city");
	}

	/**
	 * Gets the HTML output of the logging servlet.<br>
	 * <br>
	 * Possible Request Items are:<br><table>
	 * <tr><td><code>(no request)</code></td>
	 *   <td>return Frameset of logging page, start new session</td></tr>
	 * <tr><td><code>sessionId=x</code></td>    
	 *   <td>take values of session x as default</td></tr>
	 * <tr><td><code>lang=x</code></td>
	 *   <td>use language x (default is "de" for german)</td></tr>
	 * <tr><td><code>frame={head, listContent, list,content}</code></td>
	 *   <td>Requests the page for {head,listContent,list,content}-Frame</td></tr>
	 * </table><br clear="all">
	 * in head-Frame: no additional request parameters<br>
	 * in listContent-Frame:
     *   all parameters of list and conent frames. They are split and sent 
     *   to list or content frame as appropriate<br>
	 * in list-Frame:<table>
	 * <tr><td><code>sortingType=x</code></td> 
	 *   <td>sorting type indicating the data to sort after</td></tr>
	 * <tr>td><code>listSelected=x/x...</code></td>
	 *   <td>selected item, first x for first category, second x for second category
	 *   and so on</td></tr>
	 * </table><br clear="all">
	 * in Content-Frame:<br><table>
	 * <tr><td><code>dataType=x</code></td>  
	 *   <td>data type to show details to</td></tr>
	 * <tr><td><code>contentSelected=x</td>  
	 *   <td>selected data item of that type</td></tr>
	 * </table>
	 * 
	 * @param request the request object
	 * @param response the response object
	 * @throws IOException if some IO error occures
	*/
    public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        try{
        	if(debug){
            	System.out.println("QueryString: "+request.getQueryString());
            	System.out.println("Parameternames: "+request.getParameterNames().toString());
            	System.out.println("URI:         "+request.getRequestURI());
        	}

        	response.setContentType("text/html");
        	PrintWriter out = response.getWriter();

        	// get servlet data
        	ServletConfig   cfg = getServletConfig();
        	ServletContext  ctx = cfg.getServletContext();
        	ServerInterface inf = null;

			// get queries
			Hashtable queries = getQueries(request.getQueryString());
			if(debug) System.out.println("Number of elements in QueryString: "+queries.size());
			// find out session Id
			boolean isNewSession = true;
			String sessionId = null;
			// first: sessionId from queries
			if((queries.containsKey(QUERY_SESSION_ID)) && (queries.get(QUERY_SESSION_ID) != null)){
				sessionId = (String)queries.get(QUERY_SESSION_ID);
			}else
			// second: sessionId from servlet api
			if((request.getSession().getId() != null) 
				&& (request.getSession().getId().equals(""))){
				sessionId = request.getSession().getId();
			}
			if(sessionId == null || (sessions.get(sessionId)== null)){
				sessionId = generateSession(queries);
			}else{
				queries.put(QUERY_SESSION_ID, sessionId);
                isNewSession = false;	
				// session number given, and session exists:
				// set new locale, if specified
				Locale locale = getLocale((String)queries.get(QUERY_LANG));
				if(locale != null){
					((Hashtable)sessions.get(queries.get(QUERY_SESSION_ID))).put(SESSION_LOCALE, locale);
					ResourceBundle newRb 
						= java.util.ResourceBundle.getBundle("cebit/proxy/idm/localeLoggingServlet", locale);
					if(newRb != null){
						((Hashtable)sessions.get(queries.get(QUERY_SESSION_ID))).put(SESSION_RESOURCE_BUNDLE, newRb);
					}
				}
			}
			// get language ressources
			ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		
			
        	// Check, whether Server is Jigsaw with IDM
        	boolean serverOK = false;
        	if (ctx instanceof org.w3c.jigsaw.servlet.JigsawServletContext) {
            	inf = ((org.w3c.jigsaw.servlet.JigsawServletContext)ctx).getServer();

            	FramedResource fr = inf.getRoot();
            	if (fr==null) {
                	System.out.println("ERR: Root frame not found.");
            	} else {
                	String rid = fr.getIdentifier();
                	if(debug) System.out.println("INF: Root is: "+fr.getIdentifier());
                	if (!rid.equals("proxy")) {
                    	System.out.println("Root is not proxy. Error.");
                	}else{
                    	if(debug) System.out.println("INF: Root is called proxy.");
                    	ProxyFrame idm = null;
                    	idm = (ProxyFrame)fr.getFrame("cebit.proxy.idm.IDMProxyFrame");
                    	if(idm instanceof cebit.proxy.idm.IDMProxyFrame){
                        	Properties settings = Proxy.idManager.settings;
                        	try{
                            	dbi = Database.getDBI(settings);
                            	serverOK = true;
                        	}catch(Exception e){
                            	System.out.println("Exception while getting the DBI. Error.");
                        	}
                    	}
                	}
            	}
        	}
        	if (!serverOK){
            	if(debug) System.out.println("IDM-Proxy or DBI not found.");
            	out.println("<html><head><title>"
            		+rb.getString("ERROR")+"</title></head><body>");
            	out.println("<h1>+"+rb.getString("Error")+"</h1>");
            	out.println(rb.getString("NOT_RUNNING_JIGSAW"));
            	out.println("</body></html>");
        	}else{
            	if(debug) System.out.println("IDM-Proxy and DBI found.");

				// perform request
	            if(isNewSession){ 
					// output basic frameset
					getFrame(sessionId,  out);
            	
            	}else{	
	                // output specified frame or frameset
    	            if(queries.containsKey(QUERY_FRAME) && (queries.get(QUERY_FRAME) != null)){
        	            if(((String)queries.get(QUERY_FRAME)).equals("head")){
            	            getHead((String)queries.get(QUERY_SESSION_ID), out);
                	    }else
                    	if(((String)queries.get(QUERY_FRAME)).equals("listContent")){
                        	getListContent((String)queries.get(QUERY_SESSION_ID), queries, out);
                    	}else
            	        if(((String)queries.get(QUERY_FRAME)).equals("list")){
                	        getList(request, (String)queries.get(QUERY_SESSION_ID), (String)queries.get(QUERY_SORTING_TYPE),
                                (String)queries.get(QUERY_LIST_SELECTED), out);
                  	  }else
                    	if(((String)queries.get(QUERY_FRAME)).equals("content")){
                        	getContent((String)queries.get(QUERY_SESSION_ID), (String)queries.get(QUERY_DATA_TYPE),
                            	(String)queries.get(QUERY_CONTENT_SELECTED), out);
                    	}else{
                        	out.println("<html><body>"+rb.getString("BAD_REQUEST")+": "+request.getQueryString()+"</body></html>");
                    	}
                	}else{
                		// output basic frameset for given sessionId
                    	getFrame((String)queries.get(QUERY_SESSION_ID), out);
                	}
    		    }
        	}
        }catch(Exception e){
            if(debug) System.out.println("general Exception occurred: "+e.getMessage());
            e.printStackTrace();
        }
    }

    /**
     * Returns all Queries present in a query string, values accociated to names
     */
    private Hashtable getQueries(String queryString){
        String name = "";
        String value = "";
        Hashtable queries = new Hashtable();
        if(queryString == null) return queries;
        int state = 0;
        for(int i = 0; i < queryString.length(); i++){
            switch (state) {
                case 0:
                    if (queryString.charAt(i) == '='){
                        state = 1;
                    }else
                    if (queryString.charAt(i) == '&'){
                        state = 0;
                        if (name.length() > 0){
                            queries.put(name,value);
                            name = "";
                            value = "";
                        }
                    }else{
                        name = name + queryString.charAt(i);
                    }
                    break;
                case 1:
                    if (queryString.charAt(i) == '&'){
                        state = 0;
                        if (name.length() > 0){
                            queries.put(name,value);
                            name = "";
                            value = "";
                        }
                    }else{
                        value = value + queryString.charAt(i);
                    }
                    break;
                default: // should not happen
            }
        }
        if(name.length() > 0){
            queries.put(name,value);
            name = "";
            value = "";
        }
        return queries;
    }

    /**
     * Returns the Frameset for the Logging site
     */
    private void getFrame(String sessionId, PrintWriter out){
        if(debug) System.out.println("LoggingServlet.getFrame() called.");
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        // Read framefile and include sessionId
        try{
            FileInputStream fio = new FileInputStream(this.getServletContext().getRealPath("/servlet/logging/loggingFrameSet_"
             	+((Locale)((Hashtable)sessions.get(sessionId)).get(SESSION_LOCALE)).getLanguage()+".html"));
            StringBuffer sb = new StringBuffer();
            while(fio.available() > 0){
                byte[] ba = new byte[fio.available()];
                fio.read(ba);
                sb.append(new String(ba));
            }
            if(debug) System.out.println("Frameset File read.");
            int i = -1;
            while((i = sb.indexOf("sessionId=", i)) >= 0){
                sb.insert(i+10,sessionId);
                i+=10;
            }
            // write frameset out.
            out.print(sb.toString());
        }catch(FileNotFoundException fnfe){
            out.println(rb.getString("ERROR_FRAMESET_FILE_NOT_FOUND"));
        }catch(IOException ioe){
            out.println(rb.getString("ERROR_FRAMESET_FILE_NOT_READABLE"));
        }
    }

	/**
	 * Creates a new sessionId and registers it in the "sessions" Hashtable.
	 * If a locale String is given, the session will be localized using that String.
	 * On creation of a new sessionId, old sessions are checked, whether they
	 * are outdated and deleted eventually.
	 * @param locString the locales name for the session to generate 
	 * @return the sessionId of the new session
	 */
	private String generateSession(Hashtable queries){
		// create and register new session id
		String sessionId;
		if(debug) System.out.println("Generate new sessionId...");
		sessionId = Long.toString(new Random(System.currentTimeMillis()).nextLong());
		Hashtable sessionData = new Hashtable();
		sessionData.put(SESSION_LAST_TIME_USED,new Long(System.currentTimeMillis()));
		// add given values, or defaults for locale and resourceBundle
		// locale
		Locale locale = getLocale((String)queries.get(QUERY_LANG));
		ResourceBundle rb = null;
		try{
			rb 
				= java.util.ResourceBundle.getBundle("cebit/proxy/idm/localeLoggingServlet", locale);
		}catch(Exception e){
			rb = null;	
		}
		if((locale == null) || (rb == null)){
			locale = getLocale("de");
			rb = java.util.ResourceBundle.getBundle("cebit/proxy/idm/localeLoggingServlet", locale);
		}
		sessionData.put(SESSION_LOCALE, locale);
		sessionData.put(SESSION_RESOURCE_BUNDLE, rb);
		// role
		Role role = null;
		try{
			role = new Role(Integer.parseInt((String)queries.get("selectedRole")),"");
		}catch(NumberFormatException nfe){
			if(debug && ((String)queries.get("selectedRole") != null)){
				System.out.println("LoggingServlet.generateSession: Invalid "
				+"role number given:"+(String)queries.get("selectedRole"));
			}	
		}
		if(role != null){
			sessionData.put(SESSION_SELECTED_ROLE,role);
		}
		// register session
		sessions.put(sessionId, sessionData);
		// delete old sessions, if they are not used for a day
		long limit = System.currentTimeMillis() - (1000 * 60 * 60 * 24);
		Enumeration e = sessions.keys();
		while(e.hasMoreElements()){
			String id = (String)e.nextElement();
			long lastTimeUsed = ((Long)(((Hashtable)(sessions.get(id))).get(SESSION_LAST_TIME_USED))).longValue();
			if(lastTimeUsed < limit){
				sessions.remove(id);
			}
		}
		if(debug) System.out.println("Generated new sessionId: "+sessionId);
		return sessionId;
	}

    /**
     * Reads the head frame from file, and writes it out.
     * @param sessionId the session id for this session
     * @param out the output print writer
     */
    private void getHead(String sessionId, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        try{
            FileInputStream fio = new FileInputStream(getServletContext().getRealPath("/servlet/logging/loggingHead_"
            	+((Locale)((Hashtable)sessions.get(sessionId)).get(SESSION_LOCALE)).getLanguage()+".html"));;
            while(fio.available() > 0){
                byte[] ba = new byte[fio.available()];
                fio.read(ba);
                out.write(new String(ba));
            }
        }catch(FileNotFoundException fnfe){
            out.println(rb.getString("ERROR_HEAD_FILE_NOT_FOUND"));
        }catch(IOException ioe){
            out.println(rb.getString("ERROR_HEAD_FILE_NOT_READABLE"));
        }
    }

    /**
     * Reads the frame, which contains list and content frames, and gives received query parameters to them,
     * and writes it out.
     * @param sessionId the session id for this session
     * @param queries a Hashtable containing the queries for this HTTP-Request
     * @param out the output print writer
     */
    private void getListContent(String sessionId, Hashtable queries, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        try{
            FileInputStream fio = new FileInputStream(getServletContext().getRealPath("/servlet/logging/loggingListContent_"
				+((Locale)((Hashtable)sessions.get(sessionId)).get(SESSION_LOCALE)).getLanguage()+".html"));
            StringBuffer sb = new StringBuffer();
            while(fio.available() > 0){
                byte[] ba = new byte[fio.available()];
                fio.read(ba);
                sb.append(new String(ba));
            }
            if(debug) System.out.println("Frameset File read.");
            // rebuild query-string
            String query = (String)queries.get(QUERY_SESSION_ID);
            Enumeration e = queries.keys();
            while (e.hasMoreElements()){
                String key = (String)e.nextElement();
                if((!key.equals(QUERY_SESSION_ID)) && (!key.equals(QUERY_FRAME))){
                    query = query + "&"+key+"="+queries.get(key);
                }
            }
            int i = -1;
            while((i = sb.indexOf(QUERY_SESSION_ID+"=", i)) >= 0){
                sb.insert(i+10,query);
                i+=10;
            }
            // write frameset out.
            out.print(sb.toString());
        }catch(FileNotFoundException fnfe){
            out.println(rb.getString("ERROR_LISTCONTENT_FILE_NOT_FOUND"));
        }catch(IOException ioe){
            out.println(rb.getString("ERROR_LISTCONTENT_FILE_NOT_FOUND"));
        }
    }

    /**
     * Creates the sorting tree at the left hand side according to given sorting type
     * and user selection.
     */
    private void getList(HttpServletRequest request, String sessionId, String sortingType,
                         String selected, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        // get sorting type
        if(debug) System.out.println("SortingType: "+sortingType);
        if(sortingType != null){
            if((!sortingType.equals("Role-Pseudo-Trans")) &&
               (!sortingType.equals("Role-Partner-Pseudo")) &&
               (!sortingType.equals("DataName-DataValue-Partner")) &&
               (!sortingType.equals("Partner-DataName-DataValue"))){
                // use default sorting type
                sortingType = "Role-Pseudo-Trans";
            }
            ((Hashtable)sessions.get(sessionId)).put(SESSION_SORTING_TYPE,sortingType);
        }else{
            sortingType = (String)((Hashtable)sessions.get(sessionId)).get(SESSION_SORTING_TYPE);
            if(sortingType == null){
                // use default sorting type
                sortingType = "Role-Pseudo-Trans";
                ((Hashtable)sessions.get(sessionId)).put(SESSION_SORTING_TYPE,sortingType);
            }
        }
        if(debug) System.out.println("SortingType: "+sortingType);
        // build sorting array
        String sortings[] = getSortings(sortingType);
        // build selections array
        int selections[] = getSelectedIds(selected, sortings.length);

        // Output Head of list
        out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
        out.println("<html><head><meta name=\"GENERATOR\" content=\"Quanta Plus\">");
        out.println("<link rel=\"STYLESHEET\" type=\"text/css\" href=\"logging/style.css\"></head>");
        out.println("<body TOPMARGIN=\"0\" LEFTMARGIN=\"0\" MARGINWIDTH=\"0\" MARGINHEIGHT=\"0\" bgcolor=\"#efe590\">");
        out.println("<form name=\"sortingForm\" action=\"LoggingServlet\" method=\"GET\" target=\"_self\">");
        out.println("<table align=\"left\" bgcolor=\"#efe590\" border=\"0\">");
        out.println("<tr><td><strong>"+ rb.getString("LIST_GOUP_BY") + "</strong><br></td></tr>");
	out.println("<tr><td><select name=\"sortingType\" tabindex=\"1\"");
        out.println("onchange=\"document.sortingForm.submit()\">");
        // set sorting options
	out.print("<option value=\"Role-Pseudo-Trans\" ");
        if(sortingType.equals("Role-Pseudo-Trans")) out.print("selected ");
        out.println(">"+rb.getString("LIST_ROLE")+"/"
        	+ rb.getString("LIST_PSEUDONYM")+"/"
        	+ rb.getString("LIST_TRANSACTION")+"</option>");
	out.print("<option value=\"Role-Partner-Pseudo\" ");
        if(sortingType.equals("Role-Partner-Pseudo")) out.print("selected ");
        out.println(">"+rb.getString("LIST_ROLE")+"/"
        	+ rb.getString("LIST_TRANSACTION_PARTNER")+"/"
        	+ rb.getString("LIST_PSEUDONYM")+"</option>");
	out.print("<option value=\"DataName-DataValue-Partner\" ");
        if(sortingType.equals("DataName-DataValue-Partner")) out.print("selected ");
        out.print(">"+rb.getString("LIST_PERSONAL_DATA")+"/"
        	+ rb.getString("LIST_RECEIVER")+"</option>");
	out.print("<option value=\"Partner-DataName-DataValue\" ");
        if(sortingType.equals("Partner-DataName-DataValue")) out.print("selected ");
        out.print(">" + rb.getString("LIST_TRANSACTION_PARTNER")+"/"
        	+ rb.getString("LIST_TRANSMITTED_DATA")+"</option>");
        // --
        // set sessionId and target frame
	out.println("</select><input type=\"hidden\" name=\"sessionId\" value=\""+sessionId+"\">"
                    +"<input type=\"hidden\" name=\"frame\" value=\"list\"></form></td></tr>");

        // show the tree
        if(sortings[0].equals("Role")){
            showRoleList(sessionId, 0, sortings, selections, out);
        }else
        if(sortings[0].equals("DataName")){
            showDataFieldNameList(sessionId, 0, sortings, selections, out);
        }else
        if(sortings[0].equals("Pseudo")){
            showPseudonymList(sessionId, 0, sortings, selections, out);
        }else
        if(sortings[0].equals("Trans")){
            showTransactionList(sessionId, 0, sortings, selections, out);
        }else
        if(sortings[0].equals("Partner")){
            showTPartnerList(sessionId, 0, sortings, selections, out);
        }
        // output footer of page
        out.println("</table></body></html>");
    }


    /**
     * Shows a list of Roles, thereby calling a function to show a subtree for a followup category, if
     * requested.
     * @param sessionId of the current user session.
     * @param layer layer in the tree, is used for formatting the entry.
     * @param sortings string array, which states the tree structure, this is needed for structuring the output as well as
     *        for building the database queries.
     * @param selections contains the selected item for every layer of the tree. -1 states, that there is no user defined selection.
     * @param out the Printwriter for output of HTML data.
     */
    private void showRoleList(String sessionId, int layer, String[] sortings, int[] selections, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        Role[] roles = new Role[0];
        if(layer == 0){
            try{
                roles = dbi.selectRole();
            }catch(Exception e){
                out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
            }
        }else{
            // XXX wird derzeit nicht bentigt
            out.println("<td><tr>Roles in layer "+layer+" is not implemented.</td></tr>");
        }
        //Rollen auswhlen
        if(roles.length == 0){
            if(layer == 0) out.println("<tr><td>"+rb.getString("LIST_NO_DATA_AVAILABLE")+"</td></tr>");
        }else{
            // detect selected role
            // try to get selection form user input
            if(selections[layer]>=0){
                int i = 0;
                boolean found = false;
                while(i<roles.length){
                    if(roles[i].getID() == selections[layer]){
                        i = roles.length;
                        found = true;
                    }
                    i++;
                }
                if(!found){
                    for(int j = layer+1; j < selections.length; j++) selections[j] = -1;
                }
            }else
            // try to get selection from session settings
            if(((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_ROLE) != null){
                int id = ((Role)((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_ROLE)).getID();
                int i = 0;
                while(i<roles.length){
                    if(roles[i].getID() == id){
                        selections[layer]= id;
                        i = roles.length;
                    }
                    i++;
                }
            }
            // set default value, if needed
            if(selections[layer] < 0){
                selections[layer] = roles[0].getID();
            }
            // Show roles
            for(int i = 0; i < roles.length; i++){
                out.print("<tr><td>");
                // output placeholder according to layer
                for(int j = 0; j < layer; j++){
                    out.print("<img src=\"logging/treeEmpty.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"left\" border=\"0\">");
                }
                // create selection string
                String selectionString = "";
                for(int j = 0; j < layer; j++){
                    selectionString += selections[j]+"/";
                }
                // show Icon
                out.print("<img src=\"logging/treeRole.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"middle\" border=\"0\">");
                // show Role name
                if(roles[i].getID() != selections[layer]){
                    out.print("<a href=\"LoggingServlet?sessionId="+sessionId+"&frame=listContent&");
                    out.println("listSelected="+selectionString+roles[i].getID()+
                         "&dataType=Role&contentSelected="+roles[i].getID()+"\" target=\"listContent\">"+roles[i].getName()+"</a></td></tr>");
                }else{
                    // put selection into session settings
                    ((Hashtable)sessions.get(sessionId)).put(SESSION_SELECTED_ROLE,roles[i]);
                    out.println("<strong><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&"
                          +"dataType=Role&contentSelected="+roles[i].getID()+"\" target=\"content\">"+roles[i].getName()+"</a></strong></td></tr>");
                    // call subtree methods
                    if(sortings.length > layer+1){
                        if(sortings[layer+1].equals("Pseudo")){
                            showPseudonymList(sessionId, layer+1, sortings, selections, out);
                        }else
                        if(sortings[layer+1].equals("Partner")){
                            showTPartnerList(sessionId, layer+1, sortings, selections, out);
                        }
                        //XXX Other subtree functions must go here XXX
                    }
                }
            }
        }
    }


    /**
     * Shows a list of Pseudonyms, thereby calling a function to show a subtree for a followup category, if
     * requested.
     * @param sessionId of the current user session.
     * @param layer layer in the tree, is used for formatting the entry.
     * @param sortings string array, which states the tree structure, this is needed for structuring the output as well as
     *        for building the database queries.
     * @param selections contains the selected item for every layer of the tree. -1 states, that there is no user defined selection.
     * @param out the Printwriter for output of HTML data.
     */
    private void showPseudonymList(String sessionId, int layer, String[] sortings, int[] selections, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        Pseudonym[] pseudonyms = new Pseudonym[0];
        if(layer == 0){
            try{
                pseudonyms = dbi.selectPseudonym();
            }catch(Exception e){
                out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
            }
        }else{
            if(layer == 1){
                if(sortings[0].equals("Role")){
                    try{
                        pseudonyms = dbi.selectPseudonym(dbi.selectRole(selections[0]));
                        if(debug)System.out.println("Layer 1: Number of pseudonyms to show: "+pseudonyms.length);
                    }catch(Exception e){
                        out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                    }
                }else{
                    out.println("<tr><td>Pseudonyms in this layer for this sorting type are not implemented.</td></tr>");
                }
                // XXX hier knnen weitere Sortierungen bearbeitet werden
            }else
            if (layer == 2){
                if(sortings[0].equals("Role") && sortings[1].equals("Partner")){
                    try{
                        pseudonyms = dbi.selectPseudonym(dbi.selectRole(selections[0]),dbi.selectTPartner(selections[1]));
                    }catch(Exception e){
                        out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                        if(debug) e.printStackTrace();
                    }
                }else{
                    out.println("<tr><td>Pseudonyms in this layer for this sorting type are not implemented.</td></tr>");
                }
            }else{
                out.println("<tr><td>Pseudonyms in layer "+layer+" is not implemented.</td></tr>");
            }
        }

        //select pseudonyms
        if(pseudonyms.length == 0){
            if(layer == 0) out.println("<tr><td>"+rb.getString("LIST_NO_DATA_AVAILABLE")+"</td></tr>");
        }else{
            // detect selected pseudonym
            // try to get selection from user input
            if(selections[layer]>=0){
                int i = 0;
                boolean found = false;
                while(i<pseudonyms.length){
                    if(pseudonyms[i].getID() == selections[layer]){
                        i = pseudonyms.length;
                        found = true;
                    }
                    i++;
                }
                if(!found){
                    for(int j = layer+1; j < selections.length; j++) selections[j] = -1;
                }
            }else
            // try to get selection from session settings
            if(((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_PSEUDONYM) != null){
                int id = ((Pseudonym)((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_PSEUDONYM)).getID();
                int i = 0;
                while(i<pseudonyms.length){
                    if(pseudonyms[i].getID() == id){
                        selections[layer]= id;
                        i = pseudonyms.length;
                    }
                    i++;
                }
            }
            // set default value, if needed
            if(selections[layer] < 0){
                selections[layer] = pseudonyms[0].getID();
            }

            // Show pseudonyms
            if(debug) System.out.println("Number of pseudonyms to show: "+pseudonyms.length);
            for(int i = 0; i < pseudonyms.length; i++){
                // show placeholders according to layer
                out.print("<tr><td><nobr>");
                for(int j = 0; j < layer; j++){
                    out.println("<img src=\"logging/treeEmpty.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"left\" border=\"0\">");
                }
                // get pseudonym type and show icon
                PseudonymType pt[] = null;
                try{
                    pt = dbi.selectPseudonymType(pseudonyms[i]);
                }catch(Exception e){
                    out.println("Database access error while selecting pseudonym type: "+e.getMessage()+"</td></tr>");
                    pt = new PseudonymType[1];
                    pt[0].setID(-1);
                }
                String iconName = null;
                String description = null;
                // dirty hack: only use first pseudonym type
                switch (pt[0].getID()){
                    case 1: iconName = "treePseudoPers.png";   description = "Personenpseudonym"; break;
                    case 2: iconName = "treePseudoRole.png";   description = "Rollenpseudonym"; break;
                    case 3: iconName = "treePseudoRel.png";    description = "Beziehungspseudonym"; break;
                    case 4: iconName = "treePseudoRoleRel.png";description = "Rollen-Beziehungs-Pseudonym"; break;
                    case 5: iconName = "treePseudoTrans.png";  description = "Transaktionspseudonym"; break;
                }
                out.println("<img src=\"logging/"+iconName+"\" alt=\""+description+"\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"middle\" border=\"0\">");
                // create selection string
                String selectionString = "";
                for(int j = 0; j < layer; j++){
                    selectionString += selections[j]+"/";
                }
                // show pseudonym name
                if(pseudonyms[i].getID() != selections[layer]){
                    out.print("<a href=\"LoggingServlet?sessionId="+sessionId+"&frame=listContent&");
					out.println("listSelected="+selectionString+pseudonyms[i].getID()+
						 "&dataType=Pseudo&contentSelected="+pseudonyms[i].getID()+"\" target=\"listContent\">"+pseudonyms[i].getName()+"</a></nobr></td></tr>");
                }else{
                    // put selection into session settings
                    ((Hashtable)sessions.get(sessionId)).put(SESSION_SELECTED_PSEUDONYM,pseudonyms[i]);
					out.println("<strong><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&"
						  +"dataType=Pseudo&contentSelected="+pseudonyms[i].getID()+"\" target=\"content\">"+pseudonyms[i].getName()+"</a></nobr></strong></td></tr>");
                    if(sortings.length > layer+1){
                        if(sortings[layer+1].equals("Trans")){
                            showTransactionList(sessionId, layer+1, sortings, selections, out);
                        }
                        //XXX Other subtree functions must go here XXX
                    }
                }
            }
        }
    }

    /**
     * Shows a list of transactions, thereby calling a function to show a subtree for a followup category, if
     * requested.
     * @param sessionId of the current user session.
     * @param layer layer in the tree, is used for formatting the entry.
     * @param sortings string array, which states the tree structure, this is needed for structuring the output as well as
     *        for building the database queries.
     * @param selections contains the selected item for every layer of the tree. -1 states, that there is no user defined selection.
     * @param out the Printwriter for output of HTML data.
     */
    private void showTransactionList(String sessionId, int layer, String[] sortings, int[] selections, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        Transaction[] transactions = new Transaction[0];
        if(layer == 0){
            try{
                transactions = dbi.selectTransaction();
            }catch(Exception e){
                out.println("Database access error: "+e.getMessage());
            }
        }else
        if(layer > 0){
            if(sortings[layer-1].equals("Pseudo")){
                try{
                    transactions = dbi.selectTransaction(dbi.selectPseudonym(selections[layer-1]));
                }catch(Exception e){
                    out.println("Database access error: "+e.getMessage());
                }
            }else{
                // XXX wird derzeit nicht bentigt
                out.println("Transactions sind fr diese Sortierung nicht implementiert.");
            }
        }
        //find selected transaction
        if((transactions == null) || (transactions.length == 0)){
            if(layer == 0) out.println("<tr><td>"+rb.getString("LIST_NO_DATA_AVAILABLE")+"</td></tr>");
        }else{
            // detect selected transaction
            // try to get selection form user input
            if(selections[layer]>=0){
                int i = 0;
                boolean found = false;
                while(i<transactions.length){
                    if(transactions[i].getID() == selections[layer]){
                        i = transactions.length;
                        found = true;
                    }
                    i++;
                }
                if(!found){
                    for(int j = layer+1; j < selections.length; j++) selections[j] = -1;
                }
            }else
            // try to get selection from session settings
            if(((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_TRANSACTION) != null){
                int id = ((Transaction)((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_TRANSACTION)).getID();
                int i = 0;
                while(i<transactions.length){
                    if(transactions[i].getID() == id){
                        selections[layer]= id;
                        i = transactions.length;
                    }
                    i++;
                }
            }
            // set default value, if needed
            if(selections[layer] < 0){
                if(debug)if (transactions[0] == null) System.out.println("Transactions[0]=null!!!!!!!");
                selections[layer] = transactions[0].getID();
            }
            // Show transactions
            for(int i = 0; i < transactions.length; i++){
                out.print("<tr><td><nobr>");
                for(int j = 0; j < layer; j++){
                    out.print("<img src=\"logging/treeEmpty.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"left\" border=\"0\">");
                }
                // create selection string
                String selectionString = "";
                for(int j = 0; j < layer; j++){
                    selectionString += selections[j]+"/";
                }
                // show Icon
                out.print("<img src=\"logging/treeTrans.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"middle\" border=\"0\">");
                // show Transaction (time)
                if(transactions[i].getID() != selections[layer]){
                    out.print("<a href=\"LoggingServlet?sessionId="+sessionId+"&frame=listContent&");
                    out.println("listSelected="+selectionString+transactions[i].getID()+
                         "&dataType=Trans&contentSelected="+transactions[i].getID()+"\" target=\"listContent\">"+transactions[i].getStartTime().toString()+"</a></nobr></td></tr>");
                }else{
                    // put selection into session settings
                    ((Hashtable)sessions.get(sessionId)).put(SESSION_SELECTED_TRANSACTION,transactions[i]);
                    out.println("<strong><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&"
                          +"dataType=Trans&contentSelected="+transactions[i].getID()+"\" target=\"content\">"+transactions[i].getStartTime().toString()+"</a></strong></nobr></td></tr>");
                    // call subtree methods
                    if(sortings.length > layer+1){
                        //XXX Other subtree functions must go here XXX
                    }
                }
            }
        }
    }


    /**
     * Shows a list of Transaction partners, thereby calling a function to show a subtree for a followup category, if
     * requested.
     * @param sessionId of the current user session.
     * @param layer layer in the tree, is used for formatting the entry.
     * @param sortings string array, which states the tree structure, this is needed for structuring the output as well as
     *        for building the database queries.
     * @param selections contains the selected item for every layer of the tree. -1 states, that there is no user defined selection.
     * @param out the Printwriter for output of HTML data.
     */
    private void showTPartnerList(String sessionId, int layer, String[] sortings, int[] selections, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        TPartner[] tpartners = new TPartner[0];
        if(layer == 0){
            try{
                tpartners = dbi.selectTPartner();
            }catch(Exception e){
                out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                debug = true;
                if(debug) e.printStackTrace();
            }
        }else
        if(layer == 1){
            if(sortings[0].equals("Role")){
                try{
                    Transaction trans[] = dbi.selectTransaction(dbi.selectRole(selections[0]));
                    Hashtable tempPartners = new Hashtable();
                    if(debug) System.out.println(".");
                    for(int i = 0; i < trans.length; i++){
                        if(tempPartners.get(new Integer(trans[i].getPartner().getID())) == null){
                            tempPartners.put(new Integer(trans[i].getPartner().getID()),trans[i].getPartner());
                        }
                    }
                    if(debug) System.out.println("..");
                    Object  tpArray[] = tempPartners.values().toArray();
                    tpartners = new TPartner[tpArray.length];
                    for(int i = 0; i < tpArray.length; i++){
                        tpartners[i] = (TPartner)tpArray[i];
                    }
                    if(debug) System.out.println("....");
                    if(debug)System.out.println("Layer 1: Number of tpartners to show: "+tpartners.length);
                }catch(Exception e){
                    out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                    if(debug) e.printStackTrace();
                }
            }else{
                out.println("<tr><td>TPartners in this layer for this sorting type are not implemented.</td></tr>");
            }
        }else
        if(layer == 2){
            if(sortings[0].equals("DataName") && sortings[1].equals("DataValue")){
                try{
                    Transaction trans[] = dbi.selectTransaction(dbi.selectDataField(selections[1]));
                    tpartners = new TPartner[trans.length];
                    for(int i = 0; i < trans.length; i++){
                        tpartners[i] = trans[i].getPartner();
                    }
                }catch(Exception e){
                    out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                    if(debug) e.printStackTrace();
                }
            }else{
                out.println("<tr><td>TPartners in this layer for this sorting type are not implemented.</td></tr>");
            }
            // XXX hier knnen weitere Sortierungen bearbeitet werden
        }else{
            out.println("<tr><td>TPartners in layer "+layer+" is not implemented.</td></tr>");
        }

        if(debug) System.out.println(".....");
        //select tpartners
        if(tpartners.length == 0){
            if(layer == 0) out.println("<tr><td>"+rb.getString("LIST_NO_DATA_AVAILABLE")+"</td></tr>");
        }else{
            // detect selected tpartner
            // try to get selection from user input
            if(selections[layer]>=0){
                int i = 0;
                boolean found = false;
                while(i<tpartners.length){
                    if(debug)if (tpartners[i] == null) System.out.println("tpartners["+i+"] = null !!!!!");
                    if(tpartners[i].getID() == selections[layer]){
                        i = tpartners.length;
                        found = true;
                    }
                    i++;
                }
                if(!found){
                    for(int j = layer+1; j < selections.length; j++) selections[j] = -1;
                }
            }else
            // try to get selection from session settings
            if(((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_TPARTNER) != null){
                int id = ((TPartner)((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_TPARTNER)).getID();
                int i = 0;
                while(i<tpartners.length){
                    if(debug)if (tpartners[i] == null) System.out.println("tpartners["+i+"] = null !!!!!");
                    if(tpartners[i].getID() == id){
                        selections[layer]= id;
                        i = tpartners.length;
                    }
                    i++;
                }
            }
            // set default value, if needed
            if(selections[layer] < 0){
                if(debug)if (tpartners[0] == null) System.out.println("tpartners[0] = null !!!!!");
                selections[layer] = tpartners[0].getID();
            }
            if(debug) System.out.println("......");

            // Show tpartners
            if(debug) System.out.println("Number of pseudonyms to show: "+tpartners.length);
            for(int i = 0; i < tpartners.length; i++){
                // show placeholders according to layer
                out.print("<tr><td><nobr>");
                for(int j = 0; j < layer; j++){
                    out.println("<img src=\"logging/treeEmpty.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"left\" border=\"0\">");
                }
                // show icon
                out.println("<img src=\"logging/treeTPartner.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"middle\" border=\"0\">");
                // create selection string
                String selectionString = "";
                for(int j = 0; j < layer; j++){
                    selectionString += selections[j]+"/";
                }
                // show tpartner name
                if(tpartners[i].getID() != selections[layer]){
                    out.print("<a href=\"LoggingServlet?sessionId="+sessionId+"&frame=listContent&");
                    out.println("listSelected="+selectionString+tpartners[i].getID()+
                         "&dataType=Partner&contentSelected="+tpartners[i].getID()+"\" target=\"listContent\">"+tpartners[i].getName()+"</a></nobr></td></tr>");
                }else{
                    // put selection into session settings
                    ((Hashtable)sessions.get(sessionId)).put(SESSION_SELECTED_TPARTNER,tpartners[i]);
                    out.println("<strong><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&"
                          +"dataType=Partner&contentSelected="+tpartners[i].getID()+"\" target=\"content\">"+tpartners[i].getName()+"</a></strong></nobr></td></tr>");
                    if(sortings.length > layer+1){
                        if(sortings[layer+1].equals("Pseudo")){
                            showPseudonymList(sessionId, layer+1, sortings, selections, out);
                        }else
                        if(sortings[layer+1].equals("DataName")){
                            showDataFieldNameList(sessionId, layer+1, sortings, selections, out);
                        }
                        //XXX Other subtree functions must go here XXX
                    }
                }
            }
        }
    }

    /**
     * Shows a list of own Data Field Names, thereby calling a function to show a subtree for a followup category, if
     * requested.<br>
     * Caution: Data fields are handled special to some extent. There may be multiple datafields with the
     *          same name. So, under a name all corresponding datafields (shown by their name) are listed.
     *          Method showDataFieldNameList() shows the list of names, and showDataFieldValueList() shows datafield values
     *          of the corresponding datafields.
     * @param sessionId of the current user session.
     * @param layer layer in the tree, is used for formatting the entry.
     * @param sortings string array, which states the tree structure, this is needed for structuring the output as well as
     *        for building the database queries.
     * @param selections contains the selected item for every layer of the tree. -1 states, that there is no user defined selection.
     *        For Datafield names, this array contains an ID of one datafield with that name (their may be multiple fields)
     * @param out the Printwriter for output of HTML data.
     */
    private void showDataFieldNameList(String sessionId, int layer, String[] sortings, int[] selections, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        DataField[] datafields = new DataField[0];
        if(layer == 0){
            try{
                datafields = dbi.selectDataField((TPartner)null);
            }catch(Exception e){
                out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                if(debug) e.printStackTrace();
            }
        }else{
            if(layer == 1){
                if(sortings[0].equals("Category")){
                    try{
                        datafields = dbi.selectDataField((TPartner)null);
                        Vector tempdf = new Vector(10,5);
                        for(int i = 0; i < datafields.length; i++){
                            Category ct[] = dbi.selectCategory(datafields[i]);
                            boolean catFound = false;
                            int j = 0;
                            while((!catFound) && (j < ct.length)){
                                if(ct[j].getID() == selections[0]){
                                    catFound = true;
                                }
                            }
                            if(catFound) tempdf.add(datafields[i]);
                        }
                        datafields = new DataField[tempdf.size()];
                        Enumeration e = tempdf.elements();
                        int i = 0;
                        while(e.hasMoreElements()){ datafields[i] = (DataField)e.nextElement(); i++; }
                    }catch(Exception e){
                        out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                        if(debug) e.printStackTrace();
                    }
                }else
                if(sortings[0].equals("Partner")){
                    try{
                        Transaction trans[] = dbi.selectTransaction(dbi.selectTPartner(selections[0]));
                        HashSet tempDf = new HashSet();
                        for(int i = 0; i< trans.length; i++){
                            DataField df[] = dbi.selectDataField(trans[i]);
                            for(int j = 0; j < df.length; j++){
                                if(df[j].getOwner() == null){
                                    tempDf.add(df[j]);
                                }
                            }
                        }
                        datafields = (DataField[])tempDf.toArray(new DataField[tempDf.size()]);
                    }catch(Exception e){
                        out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                        if(debug) e.printStackTrace();
                    }

                }else{
                    out.println("<tr><td>Datafield names in this layer for this sorting type are not implemented.</td></tr>");
                }
                // XXX hier knnen weitere Sortierungen bearbeitet werden
            }else{
                out.println("<tr><td>Datafield names in layer "+layer+" is not implemented.</td></tr>");
            }
        }
        //select datafield
        DataField selectedField = null;
        if(datafields.length == 0){
            if(layer == 0) out.println("<tr><td>"+rb.getString("LIST_NO_DATA_AVAILABLE")+"</td></tr>");
        }else{
            // detect selected datafield (by some ID)
            // try to get selection from user input
            if(selections[layer]>=0){
                int i = 0;
                boolean found = false;
                while(i<datafields.length){
                    if(debug)if (datafields[i] == null) System.out.println("datafields["+i+"] = null !!!!!");
                    if(datafields[i].getID() == selections[layer]){
                        selectedField = datafields[i];
                        found = true;
                        i = datafields.length;
                    }
                    i++;
                }
                if(!found){
                    for(int j = layer+1; j < selections.length; j++) selections[j] = -1;
                }
            }else
            // try to get selection from session settings
            if(((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_DATA_FIELD_NAME) != null){
                int id = ((DataField)((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_DATA_FIELD_NAME)).getID();
                int i = 0;
                while(i<datafields.length){
                    if(debug)if (datafields[i] == null) System.out.println("datafields["+i+"] = null !!!!!");
                    if(datafields[i].getID() == id){
                        selections[layer]= id;
                        selectedField = datafields[i];
                        i = datafields.length;
                    }
                    i++;
                }
            }
            // set default value, if needed
            if(selections[layer] < 0){
                if(debug)if (datafields[0] == null) System.out.println("datafields[0] = null !!!!!");
                selections[layer] = datafields[0].getID();
                selectedField = datafields[0];
            }
            // put selection into session settings
            ((Hashtable)sessions.get(sessionId)).put(SESSION_SELECTED_DATA_FIELD_NAME, selectedField);

            // Remove duplicate entries from datafield name list
            Hashtable tempDfn = new Hashtable();
            Hashtable icons = new Hashtable();
            for(int i = 0; i < datafields.length; i++){
                tempDfn.put(datafields[i].getName(), datafields[i]);
                icons.put(datafields[i].getName(), "treeDataField.png");
                try{
                    if(dbi.selectTransaction(datafields[i]).length > 0){
                        icons.put(datafields[i].getName(), "treeDataFieldOut.png");
                    }
                }catch(Exception e){
                    if(debug) e.printStackTrace();
                }
            }
            DataField datafieldnames[] = (DataField[])tempDfn.values().toArray(new DataField[tempDfn.size()]);
            // Show datafield names
            if(debug) System.out.println("Number of datafield names to show: "+datafieldnames.length);
            for(int i = 0; i < datafieldnames.length; i++){
                // show placeholders according to layer
                out.print("<tr><td><nobr>");
                for(int j = 0; j < layer; j++){
                    out.println("<img src=\"logging/treeEmpty.png\" width=\"32\" height=\"32\" hspace=\"0\" "
                                +"vspace=\"0\" align=\"left\" border=\"0\">");
                }
                // show icon
                out.println("<img src=\"logging/"+icons.get(datafieldnames[i].getName())+"\" width=\"32\" height=\"32\" hspace=\"0\" "
                            +"vspace=\"0\" align=\"middle\" border=\"0\">");
                // create selection string
                String selectionString = "";
                for(int j = 0; j < layer; j++){
                    selectionString += selections[j]+"/";
                }
                // show Datafield name
                if(!datafieldnames[i].getName().equals(selectedField.getName())){
                    out.print("<a href=\"LoggingServlet?sessionId="+sessionId+"&frame=list&");
                    out.println("listSelected="+selectionString+datafieldnames[i].getID()+
                                "\">"+getP3PShortDescription(sessionId, datafieldnames[i].getName()) +"</a></nobr></td></tr>");
                }else{
                    // show selected datafield name
                    out.println("<strong>"+getP3PShortDescription(sessionId, datafieldnames[i].getName()) +"</strong></nobr></td></tr>");
                    if(sortings.length > layer+1){
                        if(sortings[layer+1].equals("DataValue")){
                            if(debug) System.out.println("Call showDataFieldValueList()");
                            showDataFieldValueList(sessionId, layer+1, sortings, selections, out);
                        }
                        //XXX Other subtree functions must go here XXX
                    }
                }
            }
        }
    }

    /**
     * Shows a list of own Data Field Values, thereby calling a function to show a subtree for a followup category, if
     * requested. <br>
     * Caution: Data fields are handled special to some extent. There may be multiple datafields with the
     *          same name. So, under a name all corresponding datafields (shown by their name) are listed.
     *          Method showDataFieldNameList() shows the list of names, and showDataFieldValueList() shows datafield values
     *          of the corresponding datafields.
     * @param sessionId of the current user session.
     * @param layer layer in the tree, is used for formatting the entry.
     * @param sortings string array, which states the tree structure, this is needed for structuring the output as well as
     *        for building the database queries.
     * @param selections contains the selected item for every layer of the tree. -1 states, that there is no user defined selection.
     * @param out the Printwriter for output of HTML data.
     */
    private void showDataFieldValueList(String sessionId, int layer, String[] sortings, int[] selections, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        DataField[] datafields = new DataField[0];
        if(debug) System.out.println("showDataFieldValueList called. layer="+layer);
        if(layer == 0){
            out.println("<tr><td>DataField Values for layer 0 are not implemented.</td></tr>");
        }else
        if(layer == 1){
            if(sortings[0].equals("DataName")){
                try{
                    datafields = dbi.selectDataField(dbi.selectDataField(selections[0]).getName());
                }catch(Exception e){
                    out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                    if(debug) e.printStackTrace();
                }
            }else{
                out.println("<tr><td>Data fields in this layer for this sorting type are not implemented.</td></tr>");
            }
        }else
        if(layer == 2){
            if(sortings[0].equals("Partner") && sortings[1].equals("DataName")){
                try{
                    TPartner partner = dbi.selectTPartner(selections[0]);
                    datafields = dbi.selectDataField(dbi.selectDataField(selections[1]).getName());
                    HashSet tempDf = new HashSet();
                    for(int i = 0; i < datafields.length; i++){
                        Transaction trans[] = dbi.selectTransaction(datafields[i]);
                        for(int j = 0; j < trans.length; j++){
                            if(trans[j].getPartner().equals(partner)){
                                tempDf.add(datafields[i]);
                                j = trans.length;
                            }
                        }
                    }
                    datafields = (DataField[])tempDf.toArray(new DataField[tempDf.size()]);
                }catch(Exception e){
                    out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                    if(debug) e.printStackTrace();
                }
            }else{
                out.println("<tr><td>Data fields in this layer for this sorting type are not implemented.</td></tr>");
            }
            // XXX hier knnen weitere Sortierungen bearbeitet werden
        }else{
            out.println("<tr><td>Datafields in layer "+layer+" is not implemented.</td></tr>");
        }
        //select datafield values
        if(datafields.length == 0){
            if(debug) System.out.println("showDataFieldNameList: keine Daten gefunden.");
            if(layer == 0) out.println("<tr><td>"+rb.getString("LIST_NO_DATA_AVAILABLE")+"</td></tr>");
        }else{
            // detect selected datafield value
            // try to get selection from user input
            if(selections[layer]>=0){
                int i = 0;
                boolean found = false;
                while(i<datafields.length){
                    if(debug)if (datafields[i] == null) System.out.println("datafields["+i+"] = null !!!!!");
                    if(datafields[i].getID() == selections[layer]){
                        i = datafields.length;
                        found = true;
                    }
                    i++;
                }
                if(!found){
                    for(int j = layer+1; j < selections.length; j++) selections[j] = -1;
                }
            }else
            // try to get selection from session settings
            if(((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_DATA_FIELD_VALUE) != null){
                int id = ((DataField)((Hashtable)sessions.get(sessionId)).get(SESSION_SELECTED_DATA_FIELD_VALUE)).getID();
                int i = 0;
                while(i<datafields.length){
                    if(debug)if (datafields[i] == null) System.out.println("datafields["+i+"] = null !!!!!");
                    if(datafields[i].getID() == id){
                        selections[layer]= id;
                        i = datafields.length;
                    }
                    i++;
                }
            }
            // set default value, if needed
            if(selections[layer] < 0){
                if(debug)if (datafields[0] == null) System.out.println("datafields[0] = null !!!!!");
                selections[layer] = datafields[0].getID();
            }

            // Show datafield values
            if(debug) System.out.println("Number of datafield names to show: "+datafields.length);
            for(int i = 0; i < datafields.length; i++){
                // show placeholders according to layer
                out.print("<tr><td><nobr>");
                for(int j = 0; j < layer; j++){
                    out.println("<img src=\"logging/treeEmpty.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"left\" border=\"0\">");
                }
                // show icon
                out.println("<img src=\"logging/treeDataFieldValue.png\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"middle\" border=\"0\">");
                // create selection string
                String selectionString = "";
                for(int j = 0; j < layer; j++){
                    selectionString += selections[j]+"/";
                }
                // show Datafield value
                String value = "<leer>";
                if((datafields[i].getValue() != null) && (datafields[i].getValue().length > 0)){
                   value = new String(datafields[i].getValue());
                }
                if(datafields[i].getID() != selections[layer]){
                    out.print("<a href=\"LoggingServlet?sessionId="+sessionId+"&frame=listContent&");
                    out.println("listSelected="+selectionString+datafields[i].getID()+
                                "&dataType=Data&contentSelected="+datafields[i].getID()+"\" target=\"listContent\">"+value+"</a></nobr></td></tr>");
                }else{
                    // put selection into session settings
                    ((Hashtable)sessions.get(sessionId)).put(SESSION_SELECTED_DATA_FIELD_VALUE,datafields[i]);
                    out.println("<strong><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&"
                               +"dataType=Data&contentSelected="+datafields[i].getID()+"\" target=\"content\">"
                               +value+"</a></strong></nobr></td></tr>");
                    if(sortings.length > layer+1){
                        if(sortings[layer+1].equals("Partner")){
                            showTPartnerList(sessionId, layer+1, sortings, selections, out);
                        }
                        //XXX Other subtree functions must go here XXX
                    }
                }
            }
        }
    }

    /**
     * Extract selection IDs from the "selected" String, which has the form
     * "a1/a2/a3/..." where aX are selection indices separated by slashes.
     * @param selected a string containing selection values for three layers
     * @return an integer array containing the extracted values, -1 for values, which were not
     *          extractable.
     */
    private int[] getSelectedIds(String selected, int layers){
        if(debug) System.out.println("SelectionString: "+selected);
        int values[] = new int[layers];
        Vector indices = new Vector(10,5);
        if(selected != null){
            int i = 0;
            while(i < selected.length()){
                int j = selected.indexOf("/",i);
                if(j <= i) j = selected.length();
                if(debug) System.out.println("i="+i+",j="+j);
                indices.add(selected.substring(i,j));
                i = j+1;
            }
        }
        for(int i = 0; i < layers; i++){
            if(i <= indices.size()){
                try{
                    if(debug) System.out.println("Sorting index "+i+": "+indices.get(i));
                    values[i] = Integer.parseInt((String)indices.get(i));
                }catch(Exception e){
                    values[i] = -1;
                }
            }else{
                values[i] = -1;
            }
        }
        if(debug){
            System.out.println("Number of selections: "+values.length);
            for(int i = 0; i < values.length; i++){
                System.out.print(values[i]+", ");
            }
            System.out.println(" ");
        }
        return values;
    }

    /**
     * Extract tree structure from a "sortingType" String, which has the form
     * "aaa/bbb/...", where xxx denotes a data type to show in the respective layer of the tree.
     * @param sortingType a string containing data types for tree layers
     * @return a string array containing the extracted substrings.
     */
    private String[] getSortings(String sortingType){
        if(debug) System.out.println("SortingType: "+sortingType);
        Vector strings = new Vector(10,5);
        int i = 0;
        while(i < sortingType.length()){
            int j = sortingType.indexOf("-",i);
            if(j <= i) j = sortingType.length();
            if (debug) System.out.println("i: "+i+", j: "+j);
            strings.add(sortingType.substring(i,j));
            i = j+1;
        }
        String values[] = new String[strings.size()];
        for(i = 0; i < strings.size(); i++){
            values[i] = (String)strings.get(i);
        }
        if(debug){
            System.out.println("Number of sortings: "+values.length);
            for(i = 0; i < values.length; i++){
                System.out.print(values[i]+", ");
            }
            System.out.println(" ");
        }

        return values;
    }

    /**
     * Creates a website containing information on an Identity Management data item and writes it to out.
     * @param sessionId the sessionId to use
     * @param dataType the Data type to show
     * @param selected the id of the selected data item
     * @param out Printwriter for html output
     */
    private void getContent(String sessionId, String dataType, String selected, PrintWriter out){
        // output page header
        out.println("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">");
        out.println("<html><head><meta name=\"GENERATOR\" content=\"Quanta Plus\">");
        out.println("<link rel=\"STYLESHEET\" type=\"text/css\" href=\"logging/style.css\"></head>");
        out.println("<body TOPMARGIN=\"10\" LEFTMARGIN=\"10\" MARGINWIDTH=\"10\" MARGINHEIGHT=\"10\" bgcolor=\"#f1f190\">");

        // find data type to show
        if((dataType == null) ||
            ((!dataType.equals("Trans")) &&
             (!dataType.equals("Partner")) &&
             (!dataType.equals("Pseudo")) &&
             (!dataType.equals("Role")) &&
             (!dataType.equals("Data")))){
             // set default value
            dataType = "none";
        }

        // get selection index
        int selections[] = getSelectedIds(selected, 1);
        int selectedIndex = selections[0];

        // call content show function
        if(dataType.equals("Role")){
            showRoleContent(sessionId, selectedIndex, out);
        }else
        if(dataType.equals("Trans")){
            showTransactionContent(sessionId, selectedIndex, out);
        }else
        if(dataType.equals("Partner")){
            showTPartnerContent(sessionId, selectedIndex, out);
        }else
        if(dataType.equals("Data")){
            showDataFieldContent(sessionId, selectedIndex, out);
        }else
        if(dataType.equals("Pseudo")){
        	showPseudonymContent(sessionId, selectedIndex, out);
        }
        // XXXX here other data type showing function can be called

        // output page footer
        out.println("</body></html>");
    }

    /**
     * Writes detailed data on the selected Role to out.
     * @param sessionId the sessionId to use
     * @param selected the ID of the Role to show
     * @param out Printwriter for html output
     */
    private void showRoleContent(String sessionId, int selected, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        /* find selected role */
        Role role = null;
        try{
            role = dbi.selectRole(selected);
        }catch(Exception e){
            out.println("Database access error: "+e.getMessage());
            if(debug) e.printStackTrace();
        }
        if(role == null){
            out.println("Role with index "+selected+" not found. Error.");
            return;
        }
        /*
         Was ist anzuzeigen:
         - Name der Rolle
         - Beschreibung?
         - Policy fr Datenherausgabe in dieser Rolle?
         - Liste der in dieser Rolle kontaktierten Transaktionspartner?
        */

       // name
       out.println("<h1><img src=\"logging/treeRole.png\" alt=\"\" width=\"32\" height=\"32\" "
                   +"hspace=\"0\" vspace=\"0\" align=\"left\" border=\"0\"> "
                   +rb.getString("LIST_ROLE")+" : "+ role.getName()+"</h1>");

       // description
       //out.println("<h2>Beschreibung</h2>");
       //out.println("nicht verfgbar");

       // Policy for disclosure of personal data
       // XXX nicht verfgbar

       // list of transaction partners contacted using this role
       Transaction transactions[] = new Transaction[0];
       try{
           transactions = dbi.selectTransaction(role);
       }catch(Exception e){
           out.println("Database access error: "+e.getMessage());
           if(debug) e.printStackTrace();
       }
       Hashtable tpartners = new Hashtable();
       for(int i = 0; i < transactions.length; i++){
           Integer id = new Integer(transactions[i].getPartner().getID());
           if(tpartners.get(id) == null){
               tpartners.put(id, transactions[i].getPartner());
            }
        }
        out.println("<h2><img src=\"logging/treeTPartner.png\" alt=\"\" width=\"32\" height=\"32\" "
                   +"hspace=\"0\" vspace=\"0\" align=\"middle\" border=\"0\"> "
                   +rb.getString("CONTENT_PARTNERS_CONTACTED_IN_ROLE")+"</h2>");
        if(tpartners.size() <= 0){
            out.println("keine.");
        }else{
            out.println("<ul>");
            Enumeration e = tpartners.elements();
            while (e.hasMoreElements()){
                TPartner tp = (TPartner)e.nextElement();
                out.println("<li><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&dataType=Partner&contentSelected="+tp.getID()+"\">"+tp.getName()+"</a></li>");
            }
            out.println("</ul>");
        }
    }

    /**
     * Writes detailed data on the selected Transaction to out.
     * @param sessionId the sessionId to use
     * @param selected the ID of the Role to show
     * @param out Printwriter for html output
     */
    private void showTransactionContent(String sessionId, int selected, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        // find selected transaction
        Transaction trans = null;
        try{
            trans = dbi.selectTransaction(selected);
        }catch(Exception e){
            out.println("Database access error: "+e.getMessage());
            if(debug) e.printStackTrace();
        }
        if(trans == null){
            out.println("Transaction with index "+selected+" not found. Error.");
            return;
        }
        /*
         Was ist anzuzeigen:
         - Inhalt/Zweck der Transaktion/Beschreibung ?
         - Transaktionspartner
         - Verwendete Rolle
         - Verwendetes Pseudonym/Pseudonymtyp
         - Zeitraum (von-bis) der Transaktion
         - Liste der an den Partner bertragenen Daten
         - Liste der vom Partner erhaltenen Daten
         - Sicherung der Datenbertragung in der Transaktion ?
        */
        // Zeit, Partner
        out.println("<h1><img src=\"logging/treeTrans.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"left\" border=\"0\"> "
                    +rb.getString("LIST_TRANSACTION")+"</h1>");
        out.println(rb.getString("CONTENT_WITH_PARTNER")+" <strong><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&"
                    +"dataType=Partner&contentSelected="+trans.getPartner().getID()+"\">"
                    +"<img src=\"logging/treeTPartner.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"middle\" border=\"0\">"+trans.getPartner().getName()+"</a></strong><br><br>");
        out.println("<strong>"+rb.getString("CONTENT_DURATION")+"</strong><br>"
        			+rb.getString("CONTENT_FROM")+" "+trans.getStartTime() + " <br>"
        			+rb.getString("CONTENT_UNTIL")+" "+trans.getStopTime()+"<br><br>");
        // Rolle
        out.println(rb.getString("CONTENT_IN_ROLE")+" <strong><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&"
                    +"dataType=Role&contentSelected="+trans.getRole().getID()+"\">"
                    +"<img src=\"logging/treeRole.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"middle\" border=\"0\">"+trans.getRole().getName()+"</a></strong><br><br>");
        // Pseudonym
        PseudonymType pt[] = null;
        try{
            pt = dbi.selectPseudonymType(trans.getPseudonym());
        }catch(Exception e){
            out.println("Database access error while selecting pseudonym type: "+e.getMessage());
            pt = new PseudonymType[1];
            pt[0].setID(-1);
        }
        String iconName = "";
        String description = "";
        // dirty hack: only use first pseudonym type
        switch (pt[0].getID()){
            case 1: iconName = "treePseudoPers.png";   description = rb.getString("CONTENT_PSEUDONYM_TYPE_PERSON"); break;
            case 2: iconName = "treePseudoRole.png";   description = rb.getString("CONTENT_PSEUDONYM_TYPE_ROLE"); break;
            case 3: iconName = "treePseudoRel.png";    description = rb.getString("CONTENT_PSEUDONYM_TYPE_RELATION"); break;
            case 4: iconName = "treePseudoRoleRel.png";description = rb.getString("CONTENT_PSEUDONYM_TYPE_ROLE_RELATION"); break;
            case 5: iconName = "treePseudoTrans.png";  description = rb.getString("CONTENT_PSEUDONYM_TYPE_TRANSACTION"); break;
        }
        out.println(rb.getString("CONTENT_A")+" <img src=\"logging/"+iconName+"\" alt=\"\" width=\"32\" height=\"32\" "
                    +"hspace=\"0\" vspace=\"0\" align=\"middle\" border=\"0\"> "+description+ " "+rb.getString("CONTENT_WAS_USED")+"<br><br>");
        //bertragene Daten
        DataField df[] = new DataField[0];
        try{
            df = dbi.selectDataField(trans);
        }catch(Exception e){
            out.println("Database access error while selecting transmitted data fields: "+e.getMessage());
            if(debug) e.printStackTrace();
        }
        int outCount = 0, inCount = 0;
        StringBuffer sbOut = new StringBuffer("<table>");
        StringBuffer sbIn = new StringBuffer("<table>");
        for(int i = 0; i < df.length; i++){
            if(df[i].getOwner() == null){
                sbOut.append("<tr><td>"+getP3PShortDescription(sessionId, df[i].getName())
                	+"</td><td>"+new String(df[i].getValue())+"</td></tr>");
                outCount ++;
            }else
            if(df[i].getOwner().equals(trans.getPartner())){
                sbIn.append("<tr><td>"+getP3PShortDescription(sessionId, df[i].getName())
                	+"</td><td>"+new String(df[i].getValue())+"</td></tr>");
                inCount ++;
            }
        }
        out.println("<h2><img src=\"logging/treeDataFieldOut.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"left\" border=\"0\"> "
                    +rb.getString("CONTENT_DATA_TRANSMITTED_TO_PARTNER")+"</h2>");
        if(outCount == 0){
            out.println(rb.getString("CONTENT_NONE"));
        }else{
            out.println(sbOut.toString());
            out.println("</table>");
        }
        out.println("<h2><img src=\"logging/treeDataFieldIn.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"left\" border=\"0\"> "
                    +rb.getString("CONTENT_DATA_RECEIVED_FROM_PARTNER")+"</h2>");
        if(inCount == 0){
            out.println(rb.getString("CONTENT_NONE"));
        }else{
            out.println(sbIn.toString());
            out.println("</table>");
        }
    }

    /**
     * Writes detailed data on the selected Transaction Partner to out.
     * @param sessionId the sessionId to use
     * @param selected the ID of the Role to show
     * @param out Printwriter for html output
     */
    private void showTPartnerContent(String sessionId, int selected, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        // find selected TPartner
        TPartner tpartner = null;
        try{
            tpartner = dbi.selectTPartner(selected);
        }catch(Exception e){
            out.println("Database access error: "+e.getMessage());
            if(debug) e.printStackTrace();
        }
        if(tpartner == null){
            out.println("TPartner with index "+selected+" not found. Error.");
            return;
        }
        /*
         Was ist anzuzeigen:
         - Bezeichnung des Partners
         - Rolle/Einordung des Partners ?
         - Pseudonym(typ) des Partners?
         - Vom Partner erhaltene Datenfelder
         - An den Partner bermittelte Datenfelder (gruppiert nach verwendetem Pseudonym)
         - Liste der Transaktionen mit diesem Partner ?
        */
        // Bezeichnung des TPartners
        out.println("<h1><img src=\"logging/treeTPartner.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"left\" border=\"0\"> "
                    + rb.getString("CONTENT_TRANSACTION_PARTNER")+" "+tpartner.getName()+"</h1>");
        // Vom partner bekannte Datenfelder
        DataField df[] = new DataField[0];
        try{
            df = dbi.selectDataField(tpartner);
        }catch(Exception e){
            out.println("Database access error while selecting datafields: "+e.getMessage());
            if(debug) e.printStackTrace();
        }
        int inCount = 0;
        StringBuffer sbIn = new StringBuffer("<table>");
        for(int i = 0; i < df.length; i++){
            sbIn.append("<tr><td>"+getP3PShortDescription(sessionId, df[i].getName())+"</td><td>"+new String(df[i].getValue())+"</td></tr>");
            inCount ++;
        }
        out.println("<h2><img src=\"logging/treeDataFieldIn.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"left\" border=\"0\"> "
                    + rb.getString("CONTENT_DATA_FROM_PARTNER")+"</h2>");
        if(inCount == 0){
            out.println(rb.getString("CONTENT_NONE"));
        }else{
            out.println(sbIn.toString());
            out.println("</table>");
        }
    }

    /**
     * Writes detailed data on the selected Data Field to out.
     * @param sessionId the sessionId to use
     * @param selected the ID of the Role to show
     * @param out Printwriter for html output
     */
    private void showDataFieldContent(String sessionId, int selected, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
        // find data field
        DataField df = null;
        try{
            df = dbi.selectDataField(selected);
        }catch(Exception e){
            out.println("Database access error: "+e.getMessage());
            if(debug) e.printStackTrace();
        }
        if(df == null){
            out.println("DataField with index "+selected+" not found. Error.");
            return;
        }
        /*
         Was ist anzuzeigen:
         - Kategorie des Datenfeldes
         - Bezeichnung des Datenfeldes
         - Inhalt des Feldes
         - Gltigkeitszeitraum (bereits abgelaufene Felder sollten farblich anders gestaltet werden)
         - Eigentmer des Feldes (nur, wenn es zu einem Partner gehrt)
         - Liste der Transaktionspartner, die dieses Datenfeld kennen (nur bei eigenen Datenfeldern)
        */
        // berschrift
        if(df.getOwner() == null){
            out.println("<h1><img src=\"logging/treeDataField.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"left\" border=\"0\"> "+getP3PShortDescription(sessionId, df.getName())+"</h1>");
        }else{
            out.println("<h1><img src=\"logging/treeDataFieldIn.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"left\" border=\"0\"> "+getP3PShortDescription(sessionId, df.getName())+"</h1>");
        }
        // Inhalt
        out.println("<big><img src=\"logging/treeDataFieldValue.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"middle\" border=\"0\">"
                    +rb.getString("CONTENT_CONTENT")
                    +" <strong>"+new String(df.getValue())+"</strong></big><br><br>");
        // Kategorien
        String[] cats = idman.dbi.P3PMapper.getCategories(df.getName());
        if(cats.length > 0){
            out.println("<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr><td align=\"top\">"
                        +"<img src=\"logging/treeCat.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                        +"vspace=\"0\" align=\"middle\" border=\"0\"></td>");
            if(cats.length == 1){
                out.println("<td>"+rb.getString("CONTENT_DATA_FIELD_BELONGS")+" "
					+rb.getString("CONTENT_TO_CATEGORY")+" <br><strong>");
            }else{
                out.println("<td>"+rb.getString("CONTENT_DATA_FIELD_BELONGS")+" "
					+rb.getString("CONTENT_TO_CATEGORIES")+" <br><strong>");
            }
            out.println(cats[0]);
            for(int i = 1; i < cats.length; i++){
                out.println(", "+cats[i]);
            }
            out.println("</strong></td></tr></table><br>");
        }
        // Beschreibung des Datenfeldes
        String[] desc = idman.dbi.P3PMapper.getDescriptions(df.getName());
        out.println("<strong>"+rb.getString("CONTENT_DATA_FIELD_DESCRIPTION")+"</strong>");
        int count = 0;
        for(int i = 0; i < desc.length; i++){
            if(!desc[(desc.length-1)-i].equals("")){
                out.print("<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\"><tr><td>");
                for(int j = 0; j < count; j++){
                    out.print("<img src=\"logging/treeEmpty.png\" alt=\"\" width=\"32\" height=\"1\" hspace=\"0\" "
                              +"vspace=\"0\" align=\"middle\" border=\"0\">");
                }
                count++;
                out.println("<img src=\"logging/treeArrow.png\" alt=\"\" width=\"16\" height=\"16\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"middle\" border=\"0\">");
                out.println("</td><td>"+desc[(desc.length-1)-i]+"</td></tr></table>");
            }
        }
        // Eigentmer
        if(df.getOwner() == null){
            out.println(rb.getString("CONTENT_DATA_FIELD_BELONGS")+" "
				+rb.getString("CONTENT_TO")+" <strong>"
				+rb.getString("CONTENT_YOUR_PERSONAL_DATA")+"</strong><br><br>");
        }else{
            out.println(rb.getString("CONTENT_DATA_FIELD_BELONGS")+" "
				+rb.getString("CONTENT_THE_PARTNER")+" <a href=\"LoggingFrame?sessionId="
                       +sessionId+"&frame=content&dataType=Parter&contentSelected="+df.getOwner().getID()+"\">"
                      + df.getOwner().getName()+"</a><br><br>");
        }
        // Gltigkeit
        out.println("<strong>"+rb.getString("CONTENT_VALIDITY_PERIOD")+"</strong><br>"
        	+rb.getString("CONTENT_FROM")+" "+df.getValidFrom()
                    + " <br>"+rb.getString("CONTENT_UNTIL")+df.getValidUntil()+"<br><br>");
        Date refDate = new Date();
        if(df.getValidFrom().after(refDate)){
            out.println(rb.getString("CONTENT_DATA_FIELD_NOT_YET_VALID"));
        }else
        if((df.getValidUntil().getTime() != 0) && df.getValidUntil().before(refDate)){
            out.println(rb.getString("CONTENT_DATA_FIELD_NOT_VALID_ANYMORE"));
        }else{
            out.println(rb.getString("CONTENT_DATA_FIELD_IS_VALID"));
        }
        // Liste von Transaktionspartnern, die dieses Datenfeld kennen (nur eigene Datenfelder)
        if(df.getOwner() == null){
            TPartner tp[] = new TPartner[0];
            try{
                Transaction trans[] = dbi.selectTransaction();
                Hashtable tempPartners = new Hashtable();
                for(int i = 0; i< trans.length; i++){
                    DataField dfs[] = dbi.selectDataField(trans[i]);
                    int j = 0;
                    while (j < dfs.length){
                        if(dfs[j].equals(df)){
                            if((tempPartners.get(new Integer(trans[i].getPartner().getID())) == null)
                               &&(dfs[j].getOwner() == null)){
                                tempPartners.put(new Integer(trans[i].getPartner().getID()), trans[i].getPartner());
                                j = dfs.length;
                            }
                        }
                        j++;
                    }
                }
                tp = new TPartner[tempPartners.size()];
                Enumeration e = tempPartners.elements();
                int i = 0;
                while (e.hasMoreElements()){
                    tp[i] = (TPartner)e.nextElement();
                    i++;
                }
            }catch(Exception e){
                out.println("<tr><td>Database access error: "+e.getMessage()+"</td></tr>");
                if(debug) e.printStackTrace();
            }
            out.println("<h2><img src=\"logging/treeDataFieldOut.png\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" "
                    +"vspace=\"0\" align=\"left\" border=\"0\">");

            if(tp.length == 0){
                out.println(rb.getString("CONTENT_DATA_FIELD_WAS_NOT_TRANSMITTED")+"</h2>");
            }else{
                out.println(rb.getString("CONTENT_DATA_FIELD_TRANSMITTED_TO")+"</h2><ul>");
                for(int i = 0; i < tp.length; i++){
                    out.println("<li><a href=\"LoggingServlet?sessionId="+sessionId+"&frame=content&"
                                +"dataType=Partner&contentSelected="+tp[i].getID()+"\">"+tp[i].getName()
                                +"</a></li>");
                }
            }
        }
    }

	/**
	 * Writes detailed data on the selected Pseudonym to out.
	 * @param sessionId the sessionId to use
	 * @param selected the ID of the Role to show
	 * @param out Printwriter for html output
	 */
	private void showPseudonymContent(String sessionId, int selected, PrintWriter out){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
		// find pseudonym
		Pseudonym ps = null;
		try{
			ps = dbi.selectPseudonym(selected);
		}catch(Exception e){
			out.println("Database access error: "+e.getMessage());
			if(debug) e.printStackTrace();
		}
		if(ps == null){
			out.println("Pseudonym with index "+selected+" not found. Error.");
			return;
		}
		/*
		 Was ist anzuzeigen:
		 - Namensinformationen zum Pseudonyminhaber
		 - Namensinformationen zum Pseudonymaussteller
		*/
		// berschrift
		Hashtable psType = getPseudonymType(sessionId, ps);
	    out.print("<h1><img src=\"logging/"+psType.get("iconName")+ "\" alt=\"\" width=\"32\" height=\"32\" hspace=\"0\" vspace=\"0\" align=\"middle\" border=\"0\">");
		out.println(psType.get("description") + "</h1>");
	    // Pseudonym name
	    out.println(rb.getString("CONTENT_NAME")+" <strong>"+ps.getName()+"</strong><br><br>");
		if(debug){ 
			System.out.println("Public Key ID: "+new String(ps.getMasterKey().getID())); 
		}	
		// Zertifikate
		PublicKey pk = null;
		psman.data.Certificate certsIssued[] = new psman.data.Certificate[0];
		psman.data.Certificate certsForKey[] = new psman.data.Certificate[0];
		try{
			// find pseudonyms public key
			pk = dbi.selectPublicKey(ps.getMasterKey().getID());
			// find certificates, which are issued by the pseudonyms key
			certsIssued = dbi.selectCertificate(pk, true);
			// find certificates, which certify the pseudonyms public key
			certsForKey = dbi.selectCertificate(pk,false);
		}catch(Exception e){
			out.println("Database access error while selecting keys and certificates.");
			if(debug) e.printStackTrace();
		}
		// remove self-certificates
		HashSet c = new HashSet();
		for(int i = 0; i < certsIssued.length; i++){
			for(int j = 0; j < certsForKey.length; j++){
				if(certsIssued[i].equals(certsForKey[j])){
					c.add(certsIssued[i]);
				}
			}
		}
		psman.data.Certificate[] selfCerts 
			= (psman.data.Certificate[])c.toArray(new psman.data.Certificate[c.size()]);  
		c.clear();
		for(int i = 0; i < certsIssued.length; i++){
			for(int j = 0; j < selfCerts.length; j++){
				if(!(selfCerts[j].equals(certsIssued[i]))){
					c.add(certsIssued[i]);
				}
			}
		}
		certsIssued = (psman.data.Certificate[])c.toArray(new psman.data.Certificate[c.size()]);
		c = new HashSet();
		for(int i = 0; i < certsForKey.length; i++){
			for(int j = 0; j < selfCerts.length; j++){
				if(!(selfCerts[j].equals(certsForKey[i]))){
					c.add(certsForKey[i]);
				}
			}
		}
		certsForKey = (psman.data.Certificate[])c.toArray(new psman.data.Certificate[c.size()]);
		// show certificates
		if(certsForKey.length == 0){
			out.println("<strong>"+rb.getString("CONTENT_PSEUDONYM_NOT_CERTIFIED")+"</strong>");
		}else{
			out.println("<h3>"+rb.getString("CONTENT_CERTIFICATES_FOR_PSEUDONYM")+"</h3>");
			out.println("<ul>");
			for(int i = 0; i < certsForKey.length; i++){
				try{
					java.security.cert.X509Certificate cert 
						= (X509Certificate)CertificateFactory.getInstance("X.509")
						.generateCertificate(new ByteArrayInputStream(certsForKey[i].getValue()));
					// show issuer and validity
					out.println(showCertificate(sessionId, cert, 9));
				}catch(CertificateException ce){
					// invalid certificate -> should not happen
					if(debug) ce.printStackTrace();
				}
			}
			out.println("</ul><br>");	
		}
		if(certsIssued.length == 0){
			out.println("<strong>"+rb.getString("CONTENT_PSEUDONYM_NOT_USED_FOR_CERTIFYING")+"</strong>");
		}else{
			out.println("<h3>"+rb.getString("CONTENT_PSEUDONYM_USED_FOR_CERTIFYING")+"</h3>");
			out.println("<ul>");
			for(int i = 0; i < certsIssued.length; i++){
				try{
					java.security.cert.X509Certificate cert 
						= (X509Certificate)CertificateFactory.getInstance("X.509")
						.generateCertificate(new ByteArrayInputStream(certsIssued[i].getValue()));
					// show recipient, content and validity	
					out.println(showCertificate(sessionId, cert, 14));
				}catch(CertificateException ce){
					// invalid certificate -> should not happen
					if(debug) ce.printStackTrace();
				}
			}
			out.println("</ul><br>");	
		}
	}
	
	/**
     * get the pseudonym type for a given pseudonym.
  	 * @param pseudonym the pseudonym to find the type for
 	 * @return a Hashtable, which consists of "iconName" and "description" for the
 	 *               pseudonym type 
 	 */
	private Hashtable getPseudonymType(String sessionId, Pseudonym pseudonym){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);		        
		PseudonymType pt[] = null;
		try{
			pt = dbi.selectPseudonymType(pseudonym);
		}catch(Exception e){
			pt = new PseudonymType[1];
			pt[0].setID(-1);
	        e.printStackTrace();
		}
		Hashtable result = new Hashtable();
		// dirty hack: only use first pseudonym type
		switch (pt[0].getID()){
			case 1: 
				result.put("iconName", "treePseudoPers.png");   
				result.put("description", rb.getString("CONTENT_PSEUDONYM_TYPE_PERSON")); 
				break;
			case 2: 
				result.put("iconName", "treePseudoRole.png");
				result.put("description", rb.getString("CONTENT_PSEUDONYM_TYPE_ROLE")); 
				break;
			case 3: 
				result.put("iconName", "treePseudoRel.png");    
				result.put("description", rb.getString("CONTENT_PSEUDONYM_TYPE_RELATION")); 
				break;
			case 4: 
				result.put("iconName", "treePseudoRoleRel.png");
			    result.put("description", rb.getString("CONTENT_PSEUDONYM_TYPE_ROLE_RELATION")); 
			    break;
			case 5: 
			    result.put("iconName", "treePseudoTrans.png");  
			    result.put("description", rb.getString("CONTENT_PSEUDONYM_TYPE_TRANSACTION")); 
			    break;
		}
		return result;		
	}
	
	/**
	 * Outputs a X.500 Distinguished Name (DN) into a HTML table.
	 * @param sessionId the sessionId of the current session
	 * @param dn the DN 
	 * @return a HTML table consisting of the fields of the DN
	 */
	private String reportDN(String sessionId, Principal dn){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);
		String s = "<table width=\"70%\" border=\"0\" bgcolor=\"white\" align=\"left\">\n";
		String dnElements[] = {"CN","O","OU","C"};
        if (debug) System.out.println("lang: "+rb.getLocale());
		String dnNames[] = {
			rb.getString("DN_NAME"),
			rb.getString("DN_ORGANIZATION"),
			rb.getString("DN_ORGANIZATION_UNIT"),
			rb.getString("DN_COUNTRY")};
			for(int i = 0; i < dnElements.length; i++){
				String content = CryptoTools.getPrincipalElement(dn,dnElements[i]);
				if(!content.equals("")){
					s += "<tr><td class=\"secgoalhead\" background=\"menu_bg.png\">"+dnNames[i]+":</td>\n";
					s += "<td class=\"secgoalother\" background=\"mitte_bg.png\">"+content+"</td></tr>\n";
				}
			}
			s += "</table>";
			return s;
	}

	/**
	 * Returns a locale specified by the given String.
	 * Currently, "de" and "en" are supported.
	 * @param locString the String specifying the locale
	 * @return the locale, or null, if locString is invalid
	 */
	Locale getLocale(String locString){
		if(locString != null){
			if(locString.equals("de")){
				return Locale.GERMAN;
			}else
			if(locString.equals("en")){
				return Locale.ENGLISH;
			}
		}
		return null;
	}
	
	/**
	 * Prints HTML-Code for showing a certificate into a String.
	 * @param sessionId the session id
	 * @param cert the certificate to show
	 * @param whatToShow an integer stating what information to show<br>
	 *        Bit 1: show issuer<br>
	 *        Bit 2: show recipient<br>
	 *        Bit 3: show content information<br>
	 *        Bit 4: show validity period<br>
	 * @return a String of HTML code for showing the requested information
	 */
	private String showCertificate(String sessionId, X509Certificate cert, int whatToShow){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);
		StringWriter outputStringWriter = new StringWriter();
		PrintWriter output = new PrintWriter(outputStringWriter);
		
		if(debug){
			output.println("Cert content:"+cert.toString()+"<br>");
		}
		// Issuer
		if((whatToShow & 1) > 0){
			output.println("<li>"+rb.getString("CONTENT_CERTIFICATE_ISSUER")+"<br>"
				+reportDN(sessionId, cert.getIssuerDN())+"<br clear=\"all\">");
		}

		// Subject (Subject DN)
		if((whatToShow & 2) > 0){
			output.println("<li>"+rb.getString("CONTENT_CERTIFICATE_ISSUED_FOR")+"<br>"
				+reportDN(sessionId, cert.getSubjectDN())+"<br clear=\"all\"><br>");
		}
			
		// Content (description)
		if((whatToShow & 4) > 0){
			output.println(rb.getString("CONTENT_ON_A_PUBLIC")
				+" "+cert.getPublicKey().getAlgorithm() +"-" 
				+ rb.getString("CONTENT_PUBLIC_KEY")+"<br><br>");
		}
			
		// Period of validity
		if((whatToShow & 8) > 0){
			output.println("<strong>"+rb.getString("CONTENT_VALIDITY_PERIOD")+"</strong><br>");
			output.println(rb.getString("CONTENT_FROM")+" "+cert.getNotBefore()+"<br>");
			output.println(rb.getString("CONTENT_UNTIL")+" "+cert.getNotBefore()+"<br>");
			output.println("</li>");
		}
		
		output.flush();
		output.close();
		return outputStringWriter.toString();
	}
	
	/**
	 * Returns the short description of an given item of the P3P naming scheme.
	 * @param sessionId the session id
	 * @param p3pName the P3P naming scheme item
	 * @return the description, or, if no description was foud, the given p3pName
	 */
	String getP3PShortDescription(String sessionId, String p3pName){
		ResourceBundle rb = (ResourceBundle)((Hashtable)sessions.get(sessionId)).get(SESSION_RESOURCE_BUNDLE);
		//TODO: use the original P3P specification (according to Language)
		if(p3pNames.contains(p3pName)){
			return rb.getString(p3pName.substring(1));
		}
		return p3pName;
	}
	
}
