package cebit.server.idm;

import java.io.*;
import java.util.*;
import java.security.cert.*;
import java.security.Principal;

import javax.servlet.*;
import javax.servlet.http.*;

import idman.mngt.*;
import idman.util.*;

import org.ssonet.mechanisms.MechanismConfiguration;
import org.ssonet.net.CryptoTools;
import org.ssonet.net.Mechanism;

public class SecInfoServlet extends HttpServlet {

	public static boolean debug = false;

	/**
	 * Ordered array of personal data items used in this servlet.
	 */
	public static final String[] pdOrder = {
		"#user.department",
		"#user.name.prefix",
		"#user.name.given",
		"#user.name.family",
		"#user.home-info.telecom.telephone.loccode",
		"#user.home-info.telecom.telephone.number",
		"#user.home-info.online.email",
		"#user.home-info.postal.street",
		"#user.home-info.postal.postalcode",
		"#user.home-info.postal.city",
		"#user.business-info.telecom.telephone.loccode",
		"#user.business-info.telecom.telephone.number",
		"#user.business-info.online.email",
		"#user.business-info.postal.street",
		"#user.business-info.postal.postalcode",
		"#user.business-info.postal.city"
	};
        
	private String debugMessage;
	private long debugTime;

	/**
	 * Locale used in this servlet
	 */
	private Locale locale;
    
    /**
     * Performs the request
     * @return the HTML-Page for the given request.
     */
    public void doGet(HttpServletRequest request,
    	HttpServletResponse response)
    	throws IOException, ServletException
    {
        response.setContentType("text/html");
        PrintWriter out = response.getWriter();
        // find out language (default is german, other language is given in QueryString with "lang=XXX"
        String lang = "de";
        if(request.getQueryString() != null){
            int langIdx = request.getQueryString().lastIndexOf("lang=");
            if(langIdx >=0){
                String s = request.getQueryString().substring(langIdx+5);
                langIdx = s.indexOf("&");
                if(langIdx >=0){
                    s = s.substring(0, langIdx);
                }
                if(!s.equals("")){
                    lang=s;
                }
            }
        }
        if(debug)System.out.println("ShopFrame: Language: "+lang);
        
        locale = Locale.GERMAN;
        if(lang.equals("en"))locale = Locale.ENGLISH;
        if(debug)System.out.println("ShopFrame: Language: "+locale.getLanguage());
        
        // generate top of the page
        String page = "";
        page += getFileContent(getFile(java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("TOP")));
        page += getFileContent(getFile(java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("MENU")));
        page += "<td background=\"../webshop/mitte_bg.png\" class=\"content\">\n";
        
        page += "<table border=\"0\"><tr><td><img src=\"../webshop/idmlogo.gif\" border=\"0\" alt=\"\"></td>\n";
        page += "<td><h2>"
        +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("DRIM_SECURITY_INFORMATION")
        +"</h2></td></tr></table>";
        // generate the site
        
        Object o = request.getAttribute("org.ssonet.context");
        if (o!=null && o instanceof idman.mngt.Context) {
            
            
            Context context = (Context)o; // Context besorgen
            if (debug) msg();
            if (debug) out("getOtherResource: getManagement()");
            if (debug) msg();
            
            // Context-Informationen ausgeben
            if (true) {
                page += "<h3>"
                +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("PSEUDONYM")
                +"</h3>\n";
                long emit = System.currentTimeMillis();
                PKI pki = new PKI(context);
                if (debug) out("getOtherResource: checkCertificate()");
                java.security.PublicKey pk = pki.checkCertificate(context.getPartnerTestCertificate());
                if (debug) msg();
                if (pk!=null) {
                    if (debug) page += "<i>"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("IS_TRUSTED_PSEUDONYM")+"\n"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("IS_CERTIFIED_BY_IDENTITY_TRUSTEE")
                    +"</i><br>";
                } else try {
                    if (debug) out("getOtherResource: expensive Database search");
                    psman.dbi.DBI dbi = idman.dbi.Database.getDBI(context);
                    psman.data.PublicKey key = context.getTPartner().getPublicKey();
                    // Alle Zertifikate des TPartners auslesen
                    psman.data.Certificate[] certs = dbi.selectCertificate(key, false);
                    // Alle Testschluessel auslesen
                    for (int i=0; pk == null && i<certs.length; i++) {
                        java.security.cert.X509Certificate cert = (X509Certificate)CertificateFactory.getInstance("X.509").generateCertificate(new ByteArrayInputStream(certs[i].getValue()));
                        pk = pki.checkCertificate(cert);
                    }
                    if (debug) msg();
                } catch (Exception e) {
                    page += "<br><i>"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("ERROR_WHILE_ACCESSING_DATABASE")
                    +e.toString()+")</i><br>";
                    e.printStackTrace();
                }
                if (pk!=null) {
                    page += "<div class=\"contenttext\">"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("PSEUDONYM_IS_CERTIFIED_BY_IDENTITY_TRUSTEE")
                    +"</div>";
                } else {
                    page += "<div class=\"contenttext\">"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("PSEUDONYM_NOT_CERTIFIED")
                    +"</div>";
                }
                if (debug) page+="<br><i>"
                +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("DECIDED_IN")
                +(System.currentTimeMillis()-emit)+ " ms.</i>";
                if (debug) out("getOtherResource: getPartnerTestCertificate()");
                java.security.cert.X509Certificate cert = context.getPartnerTestCertificate();
                if (cert!=null){
                    page +="<br><b>"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("CONTENT_OF_PSEUDONYM")
                    +"</b><br>";
                    page +="<table border=\"0\"><tr><td valign=\"top\" class=\"contenttext\">"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("ISSUED_FOR")
                    +"</td><td>"
                    +reportDN(cert.getSubjectDN())+"</td></tr>\n";
                    page += "<tr><td valign=\"top\" class=\"contenttext\">"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("ISSUER")
                    +"</td><td>"
                    +reportDN(cert.getIssuerDN())+"</td>";
                    page +="<tr><td valign=\"top\" class=\"contenttext\">"
                    +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("SERIAL_NUMBER")
                    +"</td><td class=\"secgoalother\">"
                    +cert.getSerialNumber().toString()+"</td></tr>\n";
                }
                page += "</table><br>\n";
            }
     		if (true) {
     		// Show personal data
     		page +="<h3>"
     			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("PSERONAL_DATA_TRANSMISSION")
     			+"</h3>";
     		psman.data.DataField[] sentData = {};
     		psman.data.DataField[] receivedData = {};
     		psman.data.TPartner tpartner = context.getTPartner();
     		try {
         		psman.dbi.DBI db = idman.dbi.Database.getDBI(context);
         		receivedData = db.selectDataField(tpartner);
         		java.util.Set set1 = new java.util.HashSet();
         		java.util.Date now = new java.util.Date();
         		for (int i=0; i<receivedData.length; i++) {
             		if (receivedData[i].getValidFrom().before(now) &&
                 		(receivedData[i].getValidUntil().getTime()==0 ||
                  		receivedData[i].getValidUntil().after(now))) {
                 			set1.add(receivedData[i]);
             		}
         		}
         		receivedData = (psman.data.DataField[])set1.toArray(new psman.data.DataField[set1.size()]);
         		psman.data.Transaction[] actions = db.selectTransaction(tpartner);
         		java.util.Set set2 = new java.util.HashSet();
         		for (int i=0; i<actions.length; i++) {
             		psman.data.DataField[] fields = db.selectDataField(actions[i]);
             		for (int j=0; j<fields.length; j++) {
                 		if (fields[j].getOwner()==null) set2.add(fields[j]);
             		}
         		}
         		sentData = (psman.data.DataField[])set2.toArray(new psman.data.DataField[set2.size()]);
         		if (sentData.length>0) {
             		page +="<b>"
             			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("YOU_RECEIVED_FOLLOWING_DATA")
             			+"</b>\n";
             		page +="<table width=\"70%\" border=\"0\" bgcolor=\"white\">\n";
             		for(int i = 0; i<pdOrder.length; i++){
                 		for(int c=0; c<sentData.length; c++) {
                     		if(sentData[c].getName().equals(pdOrder[i])){
                         		page +="<tr><td class=\"secgoalhead\" background=\"menu_bg.png\">"
                         			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale)
                         			.getString(sentData[c].getName().substring(1))+"</td>\n";
                         		page +="<td class=\"secgoalother\" background=\"mitte_bg.png\">"
                         			+new String(sentData[c].getValue())+"</td></tr>\n";
                     		}
                 		}
             		}
             		page +="</table><br>";
         		} else {
             		page +="<b>"
             			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("YOU_RECEIVED_NO_DATA")
             			+"</b><br><br>";
         		}
         		if (receivedData.length>0) {
             		page +="<b>"
             			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("SERVER_HAS_FOLLOWING_DATA_FROM_YOU")
             			+"</b>\n";
             		page +="<table width=\"70%\" border=\"0\" bgcolor=\"white\">\n";
             		for(int i = 0; i<pdOrder.length; i++){
                 		for(int c=0; c<receivedData.length; c++) {
                     		if(pdOrder[i].equals(receivedData[c].getName())){
                         		page +="<tr><td class=\"secgoalhead\" background=\"menu_bg.png\">"
                         			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale)
                         			.getString(receivedData[c].getName().substring(1))+"</td>\n";
                         		page +="<td class=\"secgoalother\" background=\"mitte_bg.png\">"
                         			+new String(receivedData[c].getValue())+"</td></tr>\n";
                     		}
                 		}
             		}
             		page +="</table>";
         		} else {
             		page += "<b>"
             			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("SERVER_RECEIVED_NO_DATA")
             			+"</b><br><br>";
         		}
     		} catch (Exception e) {
         		page +="<i>"
         			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("ERROR_WHILE_ACCESSING_DATABASE")
         			+e.toString()+")</i>";
     		}
     		page +="\n";
 			} // if (true) end 
/*            if (true) {
                g.append("<div id=\"actual\">");
                g.append("<h2>","Data transmission of current connection","</h2>");
                psman.data.DataField[] sentData = null;
                psman.data.DataField[] receivedData = null;
                psman.data.DataField[] allData = null;
                psman.data.Transaction transaction= context.getTransaction();
                try {
                    psman.dbi.DBI db = idman.dbi.Database.getDBI(context);
                    allData = db.selectDataField(transaction);
                    sentData = new psman.data.DataField[allData.length];
                    receivedData = new psman.data.DataField[allData.length];
                    int sent = 0;
                    int received = 0;
                    for (int i=0;i<allData.length;i++) {
                        if (allData[i].getOwner()==null) {
                            sentData[sent++]=allData[i];
                        } else {
                            receivedData[received++]=allData[i];
                        }
                    }
                    allData = sentData;
                    sentData = new psman.data.DataField[sent];
                    System.arraycopy(allData,0,sentData,0,sent);
                    allData = receivedData;
                    receivedData = new psman.data.DataField[received];
                    System.arraycopy(allData, 0, receivedData, 0, received);
 
                    if (sentData.length>0) {
                        g.append("<h3>","data actual sent","</h3><dl>");
                        for(int c=0; c<sentData.length; c++) {
                            g.append("<dt>",sentData[c].getName(),"</dt>");
                            g.append("<dd>",new String(sentData[c].getValue()),"</dd>");
                        }
                        g.append("</dl>");
                    } else {
                        g.append("No data sent.","<br>");
                    }
                    if (receivedData.length>0) {
                        g.append("<h3>","data actual received","</h3><dl>");
                        for(int c=0; c<receivedData.length; c++) {
                            g.append("<dt>",receivedData[c].getName(),"</dt>");
                            g.append("<dd>",new String(receivedData[c].getValue()),"</dd>");
                        }
                        g.append("</dl>");
                    } else {
                        g.append("no data received.","<br>");
                    }
                } catch (Exception e) {
                    g.append("<i>","Error ("+e.toString()+") occured accessing database!","</i>");
                }
                g.append("</div>\n");
            }
 */          
 			if (true) {
 			// show communication security information
     		page += "<div id=\"security\">";
     		page += "<h3>"
     			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("COMMUNICATION_SECURITY")
     			+"</h3>";
     		if (debug) out("getOtherResource: getSecurityGoalNegotiationResult()");
     		page += "<table width=\"70%\" bgcolor=\"white\"><tr>\n";
     		page += "<td background=\"menu_bg.png\"><b>"
     			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("SECURITY_GOAL")
     			+"</b></td>\n";
     		page += "<td background=\"menu_bg.png\"><img src=\"../webshop/yes_no.png\" border=\"0\" alt=\"\"></td>";
     		page += "<td background=\"menu_bg.png\"><b>"
     			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("MECHANISM")
     			+"</b></td></tr>\n<tr>\n";
     
			String yesToolTip = java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("SEC_GOAL_YES");
			String noToolTip = java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("SEC_GOAL_NO");
     
     		// accountability of the server
     		page += "<td class=\"secgoalhead\" background=\"menu_bg.png\">"
     			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("ACCOUNTABILITY_SERVER")
     			+"</td>\n"
     			+"<td class=\"secgoalother\" background=\"mitte_bg.png\">";
     		if(context.getAccountabilitySignMechanism()!=null){
         		if (debug) out("getOtherResource: getAccountabilitySignMechanism()");
         		Mechanism m = context.getAccountabilitySignMechanism();
         		if (debug) msg();
         		page += "<img src=\"../webshop/yes.png\" border=\"0\" align=\"middle\" alt=\""+yesToolTip+"\" title=\""+yesToolTip+"\"></td>"
         			+"<td class=\"secgoalother\" background=\"mitte_bg.png\">"   
         			+ reportMechanismConfiguration(m);
     		}else{
         		page += "<img src=\"../webshop/no.png\" border=\"0\" align=\"middle\" alt=\""+noToolTip+"\" title=\""+noToolTip +"\"></td>"
         			+ "<td class=\"secgoalother\" background=\"mitte_bg.png\">-"; 
     		}
     		page += "</td></tr><tr>\n";
     
			// accountability of the user
     		page += "<td class=\"secgoalhead\" background=\"menu_bg.png\">"
     			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("ACCOUNTABILITY_USER")
     			+"</td>\n"
     			+"<td class=\"secgoalother\" background=\"mitte_bg.png\">";
     		if(context.getAccountabilityAcceptMechanism()!=null){
         		if (debug) out("getOtherResource: getAccountabilityAcceptMechanism()");
         		Mechanism m = context.getAccountabilityAcceptMechanism();
         		if (debug) msg();
				page += "<img src=\"../webshop/yes.png\" border=\"0\" align=\"middle\" alt=\""+yesToolTip+"\" title=\""+yesToolTip+"\"></td>"
		   			+"<td class=\"secgoalother\" background=\"mitte_bg.png\">"   
		   			+ reportMechanismConfiguration(m);
     		}else{
				page += "<img src=\"../webshop/no.png\" border=\"0\" align=\"middle\" alt=\""+noToolTip+"\" title=\""+noToolTip +"\"></td>"
		   			+ "<td class=\"secgoalother\" background=\"mitte_bg.png\">-"; 
     		}
     		page += "</td></tr><tr>\n";
     
     		// confidentiality
     		page += "<td class=\"secgoalhead\" background=\"menu_bg.png\">"
     			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("CONFIDENTIALITY")
     			+"</td>\n"
     			+"<td class=\"secgoalother\" background=\"mitte_bg.png\">";
     		if(context.getConfidentialityMechanism()!=null){
         		if (debug) out("getOtherResource: getAccountabilityAcceptMechanism()");
         		Mechanism m = context.getConfidentialityMechanism();
         		if (debug) msg();
				page += "<img src=\"../webshop/yes.png\" border=\"0\" align=\"middle\" alt=\""+yesToolTip+"\" title=\""+yesToolTip+"\"></td>"
		   			+"<td class=\"secgoalother\" background=\"mitte_bg.png\">"   
		   			+ reportMechanismConfiguration(m);
     		}else{
				page += "<img src=\"../webshop/no.png\" border=\"0\" align=\"middle\" alt=\""+noToolTip+"\" title=\""+noToolTip +"\"></td>"
		   			+ "<td class=\"secgoalother\" background=\"mitte_bg.png\">-"; 
     		}
     		page += "</td></tr><tr>\n";
     
     		// integrity
     		page += "<td class=\"secgoalhead\" background=\"menu_bg.png\">"
     			+java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("INTEGRITY")
     			+"</td>\n"
     			+"<td class=\"secgoalother\" background=\"mitte_bg.png\">";
     		if(context.getIntegrityMechanism()!=null){
         		if (debug) out("getOtherResource: getAccountabilityAcceptMechanism()");
         		Mechanism m = context.getIntegrityMechanism();
         		if (debug) msg();
				page += "<img src=\"../webshop/yes.png\" border=\"0\" align=\"middle\" alt=\""+yesToolTip+"\" title=\""+yesToolTip+"\"></td>"
		   			+"<td class=\"secgoalother\" background=\"mitte_bg.png\">"   
		   			+ reportMechanismConfiguration(m);
     		}else{
				page += "<img src=\"../webshop/no.png\" border=\"0\" align=\"middle\" alt=\""+noToolTip+"\" title=\""+noToolTip +"\"></td>"
		   			+ "<td class=\"secgoalother\" background=\"mitte_bg.png\">-"; 
     		}
     		page += "</td></tr></table>\n";
 			} // if (true) end
		} else {
            page +="<b>"
            +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("NO_IDM_USED")
            +"</b><br>";
        }
        //generate bottom of the page
        page += getFileContent(getFile(java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("BOTTOM")));
        out.println(page);
    }
    
    private void out(String s) {
   		this.debugMessage=s;
        this.debugTime=System.currentTimeMillis();
        System.out.println(s);
    }
    private void msg() {
    	this.debugTime=System.currentTimeMillis()-debugTime;
        System.out.println(debugMessage+": "+debugTime);
    }
        
    private String reportMechanismConfiguration(Mechanism m) {
    	String mName = m.getClass().getName();
        int nStart = mName.lastIndexOf(".");
        if(nStart<0){
            mName = mName.substring(nStart+1,mName.length()-nStart);
        }
        String s = "<b>" + mName+"</b><br>";
        MechanismConfiguration mc = m.getConfiguration();
        Enumeration e = mc.keys();
        if(e.hasMoreElements()){
        	s+="<dt>"
                +java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("PARAMETER")
                +"</dt>";
        }
        while(e.hasMoreElements()){
            Object key = e.nextElement();
            Object value = mc.get(key);
            s = s + "<dd>"+key.toString()+ " = "+value.toString()+"</dd>";
        }
        return s;
    }
        
    private String reportDN(Principal dn){
        String s = "<table width=\"70%\" border=\"0\" bgcolor=\"white\">\n";
        String dnElements[] = {"CN","O","OU","C"};
        String dnNames[] = {
            java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("DN_NAME"),
            java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("DN_ORGANIZATION"),
            java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("DN_ORGANIZATION_UNIT"),
            java.util.ResourceBundle.getBundle("cebit/server/idm/locale", locale).getString("DN_COUNTRY")};
            for(int i = 0; i < dnElements.length; i++){
                String content = CryptoTools.getPrincipalElement(dn,dnElements[i]);
                if(!content.equals("")){
                    s += "<tr><td class=\"secgoalhead\" background=\"menu_bg.png\">"+dnNames[i]+"</td>\n";
                    s += "<td class=\"secgoalother\" background=\"mitte_bg.png\">"+content+"</td></tr>\n";
                }
            }
            s += "</table>";
            return s;
    }
 
    /**
     * @param name the name of the file to get.
     * @return a File for a given name, which is in the same directory as this Frame
     */
    private synchronized File getFile(String name) {
       	return new File(new File("."), "server/Jigsaw/WWW/webshop/"+name);
    }
        
    /**
     * Reads the content of a File into a String.
     * @param f the File to read
     * @return the content of the file, or an empty string if the file was not found.
     */
    private String getFileContent(File f){
        String s = "";
        try{
            FileInputStream fis = new FileInputStream(f);
            int c;
            do{
                c = fis.read();
                if(c >= 0) s += new String(new byte[] {(byte)c});
            }while(c >= 0);
            return s;
        }catch(Exception e){
            if(debug)System.err.println("File "+(f==null?"!!!null!!!":f.getName())+" not found!");
            return "";
        }
    }
        
}
