/**
 * Copyright (c) 2002 Sebastian Clau, IDM Team
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package cebit.server.idm.socket;

import org.w3c.util.LRUAble;

class IDMSocketClientState implements LRUAble {
    static final int C_IDLE = 0;	// Zombie
    static final int C_BUSY = 1;	// Is in busy list
    static final int C_FREE = 2;	// Is in free list
    static final int C_KILL = 3;	// Being killed
    static final int C_FIN  = 4;
    LRUAble next   = null;
    LRUAble prev   = null;
    IDMSocketClient  client = null;
    int     id     = 0;
    int     status = C_IDLE;
    boolean bound  = false;
    boolean marked = false;

    IDMSocketClientState csnext = null;
    IDMSocketClientState csprev = null;

    static int nextid = 0;
    static final synchronized int nextId() {
        return nextid++;
    }

    public final LRUAble getNext() {
        return next;
    }

    public final LRUAble getPrev() {
        return prev;
    }

    public final void setNext(LRUAble next) {
        this.next = next;
    }

    public final void setPrev(LRUAble prev) {
        this.prev = prev;
    }

    IDMSocketClientState(IDMSocketClientState cshead) {
        this.status   = C_IDLE;
        this.id       = nextId();
        cshead.csprev = this;
        this.csnext   = cshead;
    }

    // Used to create the head of the list.
    IDMSocketClientState() {
    }

}

