/**
 * Copyright (c) 2002 Sebastian Clau, IDM Team
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */
package cebit.server.idm.socket;

import org.w3c.tools.resources.Attribute;
import org.w3c.tools.resources.AttributeRegistry;
import org.w3c.tools.resources.IntegerAttribute;
import org.w3c.tools.resources.StringAttribute;

import org.w3c.jigsaw.http.httpd;

import org.w3c.jigsaw.config.PropertySet;

public class SocketConnectionProp extends PropertySet {
    private static String title = "Socket connection properties";

    static {
        Class     c = null;
        Attribute a = null;

        try {
          c = Class.forName("cebit.server.idm.socket.SocketConnectionProp");
        } catch (Exception ex) {
            ex.printStackTrace();
            System.exit(1);
        }
        // Minimum number of free spare threads
        a = new IntegerAttribute(IDMSocketClientFactory.MINSPARE_FREE_P
                               , new Integer(IDMSocketClientFactory.MINSPARE_FREE)
                               , Attribute.EDITABLE);
        AttributeRegistry.registerAttribute(c, a);
        // Maximum number of free spare threads
        a = new IntegerAttribute(IDMSocketClientFactory.MAXSPARE_FREE_P
                               , new Integer(IDMSocketClientFactory.MAXSPARE_FREE)
                               , Attribute.EDITABLE);
        AttributeRegistry.registerAttribute(c, a);
        // Maximum number of idle threads:
        a = new IntegerAttribute(IDMSocketClientFactory.MAXSPARE_IDLE_P
                               , new Integer(IDMSocketClientFactory.MAXSPARE_IDLE)
                               , Attribute.EDITABLE);
        AttributeRegistry.registerAttribute(c, a);
        // Maximum number of simultaneous connections:
        a = new IntegerAttribute(IDMSocketClientFactory.MAXCLIENTS_P
                               , new Integer(IDMSocketClientFactory.MAXCLIENTS)
                               , Attribute.EDITABLE);
        AttributeRegistry.registerAttribute(c, a);
        // Idle timeout for thread cache
        a = new IntegerAttribute(IDMSocketClientFactory.IDLETO_P
                               , new Integer(IDMSocketClientFactory.IDLETO)
                               , Attribute.EDITABLE);
        AttributeRegistry.registerAttribute(c, a);
        // Max number of threads
        a = new IntegerAttribute(IDMSocketClientFactory.MAXTHREADS_P
                                 , new Integer(IDMSocketClientFactory.MAXTHREADS)
                                 , Attribute.EDITABLE);
        AttributeRegistry.registerAttribute(c, a);
        // Specific address to bidn the server socket
        a = new StringAttribute(IDMSocketClientFactory.BINDADDR_P
                                , null
                                , Attribute.EDITABLE);
        AttributeRegistry.registerAttribute(c, a);
    }

    /**
     * Get this property set title.
     * @return A String giving the title for this property set.
     */

    public String getTitle() {
        return title;
    }

    SocketConnectionProp(String name, httpd server) {
        super(name, server);
    }
}
