package org.w3c.www.protocol.http ;

import java.util.Properties;
import java.net.URL;

import org.w3c.jigsaw.auth.AuthFilter;

public class HttpIDMManager extends HttpManager {

    public static boolean debug = false;

    public static synchronized HttpManager getManager(Properties p) {
        return getManager(HttpIDMManager.class, p);
    }

    protected synchronized HttpServer lookupServer(String host, int port, String role)
        throws HttpException
    {
        int    p  = (port == -1) ? 80 : port;
        String id = ((p == 80)
                     ? host.toLowerCase()+"/"+role
                     : (host.toLowerCase() +":"+p+"/"+role));
        // Check for an existing server:
        if (debug) System.out.println("HttpIDMManager.lookupServer: on "+id);
        HttpServer server = (HttpServer) servers.get(id);
        if ( server != null )
            return server;
        // Create and register a new server:
        try {
            if ("localhost".equalsIgnoreCase(host)||
                "127.0.0.1".equals(host)) {
                System.out.println(">>> localhost -> use IDM");
                server = (HttpServer) serverclass.newInstance();
            } else {
                System.out.println(">>> foreign host -> no IDM");
                server = (HttpServer) Class.forName("org.w3c.www.protocol.http.HttpBasicServer").newInstance();
            }
        } catch (Exception ex) {
            String msg = ("Unable to create an instance of \""
                          + serverclass.getName()
                          + "\", invalid config, check the "
                          + SERVER_CLASS_P + " property.");
            throw new HttpException(ex, msg);
        }
        server.initialize(this, new HttpServerState(server), host, p, timeout);
        servers.put(id, server);
        return server;
    }
    /**
     * Run the given request, in synchronous mode.
     * This method will launch the given request, and block the calling thread
     * until the response headers are available.
     * @param request The request to run.
     * @return An instance of Reply, containing all the reply
     * informations.
     * @exception HttpException If something failed during request processing.
     */

    public Reply runRequest(Request request)
        throws HttpException
    {
        Reply reply  = null;
        int   fcalls = 0;
        // Now run through the ingoing filters:
        RequestFilter filters[] = filteng.run(request);
        if ( filters != null ) {
            for (int i = 0 ; i < filters.length ; i++) {
                if ((reply = filters[fcalls].ingoingFilter(request)) != null)
                    break;
                fcalls++;
            }
        }
        // Locate the appropriate target server:
        URL target = request.getURL();
        if ( reply == null ) {
            HttpServer srv = null;
            boolean    rtry ;
            do {
                rtry = false;
                try {
                    URL proxy  = request.getProxy();
                    String role = (String)request.getState(AuthFilter.STATE_AUTHUSER);
                    if ( proxy != null )
                        srv = lookupServer(proxy.getHost(), proxy.getPort(), role);
                    else
                        srv = lookupServer(target.getHost(), target.getPort(), role);
                    request.setServer(srv);
                    reply = srv.runRequest(request);
                } catch (HttpException ex) {
                    for (int i = 0; i < fcalls; i++)
                        rtry = rtry || filters[i].exceptionFilter(request, ex);
                    if ( ! rtry )
                        throw ex;
                } finally {
                    request.unsetServer();
                }
            } while (rtry);
        }
        // Apply the filters on the way back:
        if ( filters != null ) {
            while (--fcalls >= 0) {
                Reply frep = filters[fcalls].outgoingFilter(request, reply);
                if ( frep != null ) {
                    reply = frep;
                    break;
                }
            }
        }
        return reply;
    }
}
